% Compute risk and ambiguity premium as reported in the paper
clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 3;						% Order of approximation X_hat L: V
nxhv    = 3;						% Order of approximation X_hat H: V
nev     = 3;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
beta	= 0.975;
rhoh    = 0.85;
rhol    = 0.3;
psi     = 3;
% beta    = 0.975;gam=2.0;alpha=17.75;umin= 5.297245270350528;
beta    = 0.975;gam=2.5;alpha=11.35;umin= 8.128003159701050;
% beta    = 0.975;gam=3.0;alpha= 6.65;umin=14.030403727125519;
% beta    = 0.965;gam=2.5;alpha=13.00;umin= 8.128003159701050;
ncv     = 3;                    % Order of appro    ximation C: V

filetwist   = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_paramb(par1,par2);
% sfile    = sprintf('decrules_last/resbayes_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
% sfile    = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
init_   = 1;
optim_  = 1;

paraml  = [alpha parl];
paramh  = [alpha parh];
param   = [alpha par];

%% Computation of expected values
load(filetwist);
C=data.xd(:,1);
Xl=data.xd(:,2);
Xh=data.xd(:,3);
E=data.xd(:,4);

WCX     = data.wx;
y       = data.y1d;
w       = data.w1d;
INT3    = data.y3d;
wint3   = data.w3d;
Kh      = data.kh;
Kl      = data.kl;
alpha   = param(1);
beta    = param(2);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
spl     = param(17);
sph     = param(18);
coefxl  = param(19);
coefxh  = param(20);
coefc   = param(21);
coefe   = param(22);
accprob = param(23);
gam     = param(24);
Kh      = data.kh;
Kl      = data.kl;
Fh      = data.fh;
Fl      = data.fl;
spi     = sqrt(pi);
spi3    = spi*spi*spi;
nint    = size(w,1);
    nint3   = size(INT3,1);
outbl    = 0;
outbh    = 0;
outcl    = 0;
outch    = 0;
outtl    = 0;
outth    = 0;
for i=1:nint
    
    yi  = y(i);
    wi  = w(i);
    innerbl= 0;
    innerbh= 0;
    innercl= 0;
    innerch= 0;
    innertl= 0;
    innerth= 0;    
    for j=1:nint3
        gpl     = ggl+rhol*(Xl+spl*yi)+sgl*INT3(j,2)+sxl*INT3(j,1);
        gph     = ggh+rhoh*(Xh+sph*yi)+sgh*INT3(j,2)+sxh*INT3(j,1);
        gdpl    = gdl+psil*rhol*(Xl+spl*yi)+sdl*INT3(j,3)+psil*sxl*INT3(j,1);
        gdph    = gdh+psih*rhoh*(Xh+sph*yi)+sdh*INT3(j,3)+psih*sxh*INT3(j,1);
        %
        % nupij: i-> subscript; j->superscript
        %
        nupll   = [gpl-ggl-rhol*Xl gdpl-gdl-psil*rhol*Xl];
        nuphh   = [gph-ggh-rhoh*Xh gdph-gdh-psih*rhoh*Xh];
        nuphl   = [gpl-ggh-rhoh*Xh gdpl-gdh-psih*rhoh*Xh];
        nuplh   = [gph-ggl-rhol*Xl gdph-gdl-psil*rhol*Xl];
        %
        % Compute the etas
        %
        L1      = likelihood(nupll,Fl);
        L2      = likelihood(nuphl,Fh);
        etapl   = E.*L1./(E.*L1+(1-E).*L2);

        L1      = likelihood(nuplh,Fl);
        L2      = likelihood(nuphh,Fh);
        etaph   = E.*L1./(E.*L1+(1-E).*L2);

        zxll    = (rhol*Xl+nupll*Kl')*coefxl;
        zxlh    = (rhol*Xl+nuplh*Kl')*coefxl;
       
        zxhl    = (rhoh*Xh+nuphl*Kh')*coefxh;
        zxhh    = (rhoh*Xh+nuphh*Kh')*coefxh;

        zel     = norm_inv(etapl)*coefe;
        zeh     = norm_inv(etaph)*coefe;

        zcl     = (C+gpl)*coefc;
        zch     = (C+gph)*coefc;
        
        Tvl     = makepoly4_v([zcl,zxll,zxhl,zel]);
        Tvl2    = makepoly3_v([zcl,zxll,zxhl]);

        Tvh     = makepoly4_v([zch,zxlh,zxhh,zeh]);
        Tvh2    = makepoly3_v([zch,zxlh,zxhh]);

        vtpl                    = exp(Tvl*thv);
        vtpl(etapl<accprob)     = exp(Tvl2(etapl<accprob,:)*thvh);
        vtpl(etapl>(1-accprob)) = exp(Tvl2(etapl>(1-accprob),:)*thvl);
        
        vtph                    = exp(Tvh*thv);
        vtph(etaph<accprob)     = exp(Tvh2(etaph<accprob,:)*thvh);
        vtph(etaph>(1-accprob)) = exp(Tvh2(etaph>(1-accprob),:)*thvl);

        innertl  = innertl+vtpl*wint3(j)/spi3;
        innerth  = innerth+vtph*wint3(j)/spi3;
        

    end
        outtl    = outtl+exp(-alpha*innertl)*wi/spi;
        outth    = outth+exp(-alpha*innerth)*wi/spi;
end
tmp =(1-beta)*(exp((1-gam)*C)/(1-gam)+umin)-beta*log(E.*outtl+(1-E).*outth)/alpha;
ECt = ((1-gam)*(tmp-umin)).^(1/(1-gam));
fprintf('ECt = %12.8f\n',mean(ECt))
