% This code replicates the results of Table 10 in the paper:
% It computes the gamma* needed to get the same certainty equivalent 
% as in the Ambiguity aversion model in the Bayesian model. It first
% requires that the file Certainty_equivalent_twisted.m be run for the case
% under study to obtain the number Ect (Certainty equivalent in twisted case).
% The user then needs to provide this number in the code. This is achieved on line 49
% in the XX matrix.
clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 3;						% Order of approximation X_hat L: V
nxhv    = 3;						% Order of approximation X_hat H: V
nev     = 3;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
beta        = 0.975;
rhoh        = 0.85;
rhol        = 0.3;
psi         = 3;
ECt=[                         % gamma* we found
0.975 2.0   0.702981375425611 % 3.48793
0.975 2.5   0.531741353045299 % 3.51117
0.975 3.0   0.437986788590451 % 3.52169
0.965 2.5   0.567215902642970 % 3.76459
   ];
iexp        = 2;
beta        = ECt(iexp,1);
gam         = ECt(iexp,2);
CEt         = ECt(iexp,3);
ncv         = 3;                    % Order of approximation C: V
gam         = 3.51117;  % gamma* for the case 1

filetwist   = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_paramb(par1,par2);
sfile    = sprintf('decrules_last/resbayes_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
% sfile    = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
init_   = 1;
lfile   = 'tmpres';
optim_  = 1;
%% Initial conditions
if init_;
    thvl01  = 1.5;
    thvh01  = 1.5;
    thv01   = 1.1;
    
    thvl0   = zeros(size(dats.tv,2),1);
    thvl0(1)= thvl01;
    thcl0   = thvl0;
    thbl0   = thvl0;
    thvh0   = zeros(size(dats.tv,2),1);
    thvh0(1)= thvh01;
    thvh0(2)=-0.2;
    thch0   = thvh0;
    thbh0   = thvh0;
    
    thv0    = zeros(size(data.tv3,2),1);
    thv0(1) = thv01;
    thv0(2) = -0.2;
    thc0    = thv0;
    thb0    = thv0;
else    
    load(lfile)
    tmp     = dats.tv0*thvl;
    thvl0   = dats.tv\tmp;
    tmp     = dats.tv0*thvh;
    thvh0   = dats.tv\tmp;
    tmp     = data.tv40*thv;
    thv0    = data.tv4\tmp;
end
% load tmpres
% thbl0   = thbl;
% thbh0   = thbh;
% thb0    = thb;
paraml  = [alpha parl];
paramh  = [alpha parh];
param   = [alpha par];
paramcl = paraml;
paramcl(end)=0;
paramch = paramh;
paramch(end)=0;
paramc  = param;
paramc(end)=0;

%% Solve for the low rho case
thbl    = fcsolve(@residl_v,thbl0,options,paraml,dats);
%% Solve for the high rho case
thbh    = fcsolve(@residh_v,thbh0,options,paramh,dats);
%% Main model
data.thvl= thbl;
data.thvh= thbh;
thb     = fcsolve(@resid_v,thb0,options,param,data);
%% Book keeping
save('tmpres','thbl','thbh','thb');%,'thcl','thch','thc');
%% Computation of expected values
% load(filetwist);
C=data.xd(:,1);
Xl=data.xd(:,2);
Xh=data.xd(:,3);
E=data.xd(:,4);
umin=8.1280;
WCX     = data.wx;
y       = data.y1d;
w       = data.w1d;
INT3    = data.y3d;
wint3   = data.w3d;
Kh      = data.kh;
Kl      = data.kl;
alpha   = param(1);
beta    = param(2);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
spl     = param(17);
sph     = param(18);
coefxl  = param(19);
coefxh  = param(20);
coefc   = param(21);
coefe   = param(22);
accprob = param(23);
gam     = param(24);
Kh      = data.kh;
Kl      = data.kl;
Fh      = data.fh;
Fl      = data.fl;
spi     = sqrt(pi);
spi3    = spi*spi*spi;
nint    = size(w,1);
    nint3   = size(INT3,1);
outbl    = 0;
outbh    = 0;
outcl    = 0;
outch    = 0;
outtl    = 0;
outth    = 0;
for i=1:nint
    
    yi  = y(i);
    wi  = w(i);
    innerbl= 0;
    innerbh= 0;
    innercl= 0;
    innerch= 0;
    innertl= 0;
    innerth= 0;    
    for j=1:nint3
        gpl     = ggl+rhol*(Xl+spl*yi)+sgl*INT3(j,2)+sxl*INT3(j,1);
        gph     = ggh+rhoh*(Xh+sph*yi)+sgh*INT3(j,2)+sxh*INT3(j,1);
        gdpl    = gdl+psil*rhol*(Xl+spl*yi)+sdl*INT3(j,3)+psil*sxl*INT3(j,1);
        gdph    = gdh+psih*rhoh*(Xh+sph*yi)+sdh*INT3(j,3)+psih*sxh*INT3(j,1);
        %
        % nupij: i-> subscript; j->superscript
        %
        nupll   = [gpl-ggl-rhol*Xl gdpl-gdl-psil*rhol*Xl];
        nuphh   = [gph-ggh-rhoh*Xh gdph-gdh-psih*rhoh*Xh];
        nuphl   = [gpl-ggh-rhoh*Xh gdpl-gdh-psih*rhoh*Xh];
        nuplh   = [gph-ggl-rhol*Xl gdph-gdl-psil*rhol*Xl];
        %
        % Compute the etas
        %
        L1      = likelihood(nupll,Fl);
        L2      = likelihood(nuphl,Fh);
        etapl   = E.*L1./(E.*L1+(1-E).*L2);

        L1      = likelihood(nuplh,Fl);
        L2      = likelihood(nuphh,Fh);
        etaph   = E.*L1./(E.*L1+(1-E).*L2);

        zxll    = (rhol*Xl+nupll*Kl')*coefxl;
        zxlh    = (rhol*Xl+nuplh*Kl')*coefxl;
       
        zxhl    = (rhoh*Xh+nuphl*Kh')*coefxh;
        zxhh    = (rhoh*Xh+nuphh*Kh')*coefxh;

        zel     = norm_inv(etapl)*coefe;
        zeh     = norm_inv(etaph)*coefe;

        zcl     = (C+gpl)*coefc;
        zch     = (C+gph)*coefc;
        
        Tbl     = makepoly3_vb([zxll,zxhl,zel]);
        Tbl2    = makepoly2_vb([zxll,zxhl]);

        Tbh     = makepoly3_vb([zxlh,zxhh,zeh]);
        Tbh2    = makepoly2_vb([zxlh,zxhh]);
        
        vbpl                    = exp(Tbl*thb+(1-gam)*gpl);
        vbpl(etapl<accprob)     = exp(Tbl2(etapl<accprob,:)*thbh+(1-gam)*gpl(etapl<accprob));
        vbpl(etapl>(1-accprob)) = exp(Tbl2(etapl>(1-accprob),:)*thbl+(1-gam)*gpl(etapl>(1-accprob),:));
        
        vbph                    = exp(Tbh*thb+(1-gam)*gph);
        vbph(etaph<accprob)     = exp(Tbh2(etaph<accprob,:)*thbh+(1-gam)*gpl(etaph<accprob));
        vbph(etaph>(1-accprob)) = exp(Tbh2(etaph>(1-accprob),:)*thbl+(1-gam)*gpl(etaph>(1-accprob),:));
        
        innerbl  = innerbl+vbpl*wint3(j)/spi3;
        innerbh  = innerbh+vbph*wint3(j)/spi3;

    end
        outbl    = outbl+innerbl*wi/spi;
        outbh    = outbh+innerbh*wi/spi;
end
ECb	= ((1-gam)*exp((1-gam)*C).*((1-beta)/(1-gam)-beta*(E.*outbl+(1-E).*outbh))).^(1/(1-gam));
fprintf('Gamma*=%6.4f, CE_t=%6.4f, CE_b=%6.4f\n',gam,CEt,mean(ECb))
