function [data,dats,param,paraml,paramh]   = build_paramb(param1,param2)

gam     = param1(1);
rhoh    = param1(2);
rhol    = param1(3);
psi     = param1(4);
beta    = param1(5);

ncv     = param2(1);
nxlv    = param2(2);
nxhv    = param2(3);
nev     = param2(4);
ncr     = param2(5);
nxlr    = param2(6);
nxhr    = param2(7);
ner     = param2(8);
ncp     = param2(9);
nxlp    = param2(10);
nxhp    = param2(11);
nep     = param2(12);

nodesc  = param2(13);
nodesxl = param2(14);
nodesxh = param2(15);
nodese  = param2(16);

nca     = param2(17);
nxla    = param2(18);
nxha    = param2(19);
nea     = param2(20);

%
% Prepare functions that compute polynomials
%
delete('sol_functions/makepoly30_vb.m')
delete('sol_functions/makepoly20_vb.m')
delete('sol_functions/makepoly3_vb.m')
delete('sol_functions/makepoly2_vb.m')
folder='sol_functions/';
write_poly('makepoly30_vb',[2,2,2],folder);  % Function for V
write_poly('makepoly20_vb',[2,2],folder);      % Function for V
%
% Prepare functions that compute polynomials
%
write_poly('makepoly3_vb',[nxlv,nxhv,nev],folder);  % Function for V
write_poly('makepoly2_vb',[nxlv,nxhv],folder);      % Function for V
write_poly('makepoly3_pb',[nxlp,nxhp,nep],folder);  % Function for V
write_poly('makepoly2_pb',[nxlp,nxhp],folder);      % Function for V
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%    Algorithm Parameters + Preparation of polynomial and Integration     %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
zc          = gauss_herm(nodesc);
[zxl,wxl]   = gauss_herm(nodesxl);      % Interpolation nodes Xl
[zxh,wxh]   = gauss_herm(nodesxh);      % Interpolation nodes Xh
[ze,we]     = gauss_herm(nodese);       % Interpolation nodes e

tmpxl       = kron(zxl,ones(nodesxh*nodese,1));
tmpxh       = kron(ones(nodesxl,1),kron(zxh,ones(nodese,1)));
tmpe        = kron(ones(nodesxl*nodesxh,1),ze);

Zmatrix     = [tmpxl tmpxh tmpe];
clear tmpxl tmpxh tmpe;

data.tv30   = makepoly30_vb(Zmatrix);             % Polynomial for V

data.tv3    = makepoly3_vb(Zmatrix);             % Polynomial for V
data.tv2    = makepoly2_vb(Zmatrix(:,1:2));      % Polynomial for V
data.tp3    = makepoly3_pb(Zmatrix);             % Polynomial for V
data.tp2    = makepoly2_pb(Zmatrix(:,1:2));      % Polynomial for V

tmpc        = ones(nodesxl*nodesxh,1);
tmpxl       = kron(zxl,ones(nodesxh,1));
tmpxh       = kron(ones(nodesxl,1),zxh);
Zmatrix     = [tmpxl tmpxh];
clear tmpxl tmpxh;

dats.tv0    = makepoly20_vb(Zmatrix);      % Polynomial for V
dats.tv     = makepoly2_vb(Zmatrix);             % Polynomial for V
dats.tp     = makepoly2_pb(Zmatrix);             % Polynomial for V

dats.wx     = kron(wxl,wxh);           % Integration for residuals
data.wx     = kron(wxl,kron(wxh,we));  % Integration for residuals
integ       = integ_data(1);                    % Integration wrt shocks
% If parameter=1 -> Mono
% If parameter=0 -> Gauss-Hermite
data.y1d    = integ.y1d;
data.w1d    = integ.w1d;
data.y3d    = integ.y3d;
data.w3d    = integ.w3d;

dats.y1d    = integ.y1d;
dats.w1d    = integ.w1d;
dats.y3d    = integ.y3d;
dats.w3d    = integ.w3d;

accprob     = 0.001;
tmp         = gauss_herm(4);
coefc       = zc(1)/tmp(1);                     % Scaling coefficient for C
zx10        = gauss_herm(10);
coefxl      = 350*zxl(1)/zx10(1);               % Scaling coefficient for X_hat - low rho
coefxh      = 40*zxh(1)/zx10(1);                % Scaling coefficient for X_hat - high rho
coefe       = ze(1)/zx10(1);                    % Scaling coefficient for X_hat - high rho
C           = zc/coefc;                         % Builds C
XL          = zxl/coefxl;                       % Builds X_hat - low rho
XH          = zxh/coefxh;                       % Builds X_hat - high rho
E           = norm_cdf(ze/coefe);               % Builds E

data.x      = [kron(XL,ones(nodesxl*nodese,1)) kron(ones(nodesxl,1),kron(XH,ones(nodese,1))) kron(ones(nodesxl*nodesxh,1),E)];
dats.x      = [kron(XL,ones(nodesxh,1)) kron(ones(nodesxl,1),XH)];
data.wx     = kron(wxl,kron(wxh,we));
dats.wx     = kron(wxl,wxh);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%            Historical Data + Invariant Structural Parameters            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
psih    = psi;                      % dividend magnification parameter
psil    = psi;                      % dividend magnification parameter
ggl     = 0.01925200;               % Average consumption growth - low rho
ggh     = 0.01925200;               % Average consumption growth - high rho
gdl     = 0.02319300;               % Average dividend growth - low rho
gdh     = 0.02319300;               % Average dividend growth - high rho

if rhoh==0.85
    if psi==3
        if rhol==0.2
            data_psi300_rhol20_rhoh85;
            sgl     = sqrt(0.00050994);
            sgh     = sqrt(0.00025898);
            sdl     = sqrt(0.04484000);
            sdh     = sqrt(0.04755800);
            sxl     = sqrt(0.00054769);
            sxh     = sqrt(0.00051808);
        elseif rhol==0.25
            data_psi300_rhol25_rhoh85;
            sgl     = sqrt(0.00048564);
            sgh     = sqrt(0.00025898);
            sdl     = sqrt(0.04498100);
            sdh     = sqrt(0.04755800);
            sxl     = sqrt(0.00054584);
            sxh     = sqrt(0.00051808);
        else
            data_psi300_rhol30_rhoh85;
            sgl     = sqrt(0.00046405);
            sgh     = sqrt(0.00025898);
            sdl     = sqrt(0.04515900);
            sdh     = sqrt(0.04755800);
            sxl     = sqrt(0.00054077);
            sxh     = sqrt(0.00051808);
        end
    elseif psi==2.5
        if rhol==0.3
            data_psi250_rhol30_rhoh85;
            sgl     = sqrt(0.00046659);
            sgh     = sqrt(0.00026453);
            sdl     = sqrt(0.04646400);
            sdh     = sqrt(0.04808400);
            sxl     = sqrt(0.00053700);
            sxh     = sqrt(0.00050760);
        elseif rhol==0.25
            data_psi250_rhol25_rhoh85;
            sgl     = sqrt(0.00048904);
            sgh     = sqrt(0.00026453);
            sdl     = sqrt(0.04636600);
            sdh     = sqrt(0.04808400);
            sxl     = sqrt(0.00054111);
            sxh     = sqrt(0.00050760);
        end
    end
else
    if rhol==0.2
        data_psi300_rhol20_rhoh90;
        sgl     = sqrt(0.00050994);
        sgh     = sqrt(0.00019836);
        sdl     = sqrt(0.04484000);
        sdh     = sqrt(0.04730800);
        sxl     = sqrt(0.00054769);
        sxh     = sqrt(0.00059686);
    elseif rhol==0.25
        data_psi300_rhol25_rhoh90;
        sgl     = sqrt(0.00048564);
        sgh     = sqrt(0.00019836);
        sdl     = sqrt(0.04498100);
        sdh     = sqrt(0.04730800);
        sxl     = sqrt(0.00054584);
        sxh     = sqrt(0.00059686);
    else
        data_psi300_rhol30_rhoh90;
        sgl     = sqrt(0.00046405);
        sgh     = sqrt(0.00019836);
        sdl     = sqrt(0.04515900);
        sdh     = sqrt(0.04730800);
        sxl     = sqrt(0.00054077);
        sxh     = sqrt(0.00059686);
    end
end
whole   = 1930:2011;
t0      = 1977;
t1      = 2011;
smpl    = find(whole==t0):find(whole==t1);
Xld     = DATX(smpl,2);
Xhd     = DATX(smpl,3);
Ed      = DATX(smpl,4);  %(Proba of rhol)
Cd      = log(DATX(smpl,5)/10000);

data.xd = [Cd(2:end) Xld(2:end) Xhd(2:end) Ed(2:end)];
dats.xd = [Cd(2:end) Xld(2:end) Xhd(2:end)];
data.xpd= [Cd Xld Xhd Ed];
dats.xpd= [Cd Xld Xhd];

Zmatrix     = [Cd(2:end)*coefc Xld(2:end)*coefxl Xhd(2:end)*coefxh norm_inv(Ed(2:end))*coefe];
data.tv3d   = makepoly3_vb(Zmatrix);             % Polynomial for V
data.tv2d   = makepoly2_vb(Zmatrix(:,1:2));      % Polynomial for V
dats.tvd    = makepoly2_vb(Zmatrix(:,1:2));      % Polynomial for V
data.tp3d   = makepoly3_pb(Zmatrix);             % Polynomial for V
data.tp2d   = makepoly2_pb(Zmatrix(:,1:2));      % Polynomial for V
dats.tpd    = makepoly2_pb(Zmatrix(:,1:2));      % Polynomial for V

Zmatrix     = [Cd*coefc Xld*coefxl Xhd*coefxh norm_inv(Ed)*coefe];
data.tpp3d   = makepoly3_pb(Zmatrix);             % Polynomial for V
data.tpp2d   = makepoly2_pb(Zmatrix(:,1:2));      % Polynomial for V
dats.tppd    = makepoly2_pb(Zmatrix(:,1:2));      % Polynomial for V
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                            Kalman Filter                                %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Computation of Pl and Fl
%
crit    = 1;
tol     = 1e-12;
P0      = 1;
while crit>tol
    Fl  = [P0+sgl*sgl psil*P0;psil*P0 psil*psil*P0+sdl*sdl];
    IF  = chol(Fl);
    IF  = IF\eye(2);
    IF  = IF*IF';
    Pl  = rhol*rhol*P0-rhol*rhol*P0*P0*([1 psil]*IF*[1 psil]')+sxl*sxl;
    crit= abs(Pl-P0);
    P0  = Pl;
end
spl     = sqrt(Pl);
data.fl = Fl;
data.kl = rhol*Pl*([1 psil]*IF);
dats.kl = rhol*Pl*([1 psil]*IF);
%
% Computation of Ph and Fh
%
crit    = 1;
tol     = 1e-12;
P0      = 1;
while crit>tol
    Fh  = [P0+sgh*sgh psih*P0;psih*P0 psih*psih*P0+sdh*sdh];
    IF  = chol(Fh);
    IF  = IF\eye(2);
    IF  = IF*IF';
    Ph  = rhoh*rhoh*P0-rhoh*rhoh*P0*P0*([1 psih]*IF*[1 psih]')+sxh*sxh;
    crit= abs(Ph-P0);
    P0  = Ph;
end
sph     = sqrt(Ph);
data.fh = Fh;
data.kh = rhoh*Ph*([1 psih]*IF);
dats.kh = rhoh*Ph*([1 psih]*IF);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                                Utility                                  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

U4      = exp((1-gam)*data.x(:,1))/(1-gam);
U3      = exp((1-gam)*dats.x(:,1))/(1-gam);
Ud      = exp((1-gam)*data.xd(:,1))/(1-gam);
Umin    = 0;
if gam>1;
    CM      = 1.01*data.x(1,1);
    Umin    = abs(exp((1-gam)*CM)/(1-gam));
end
data.umin= Umin;
data.u  = U4+Umin;
data.ud = Ud+Umin;
dats.u  = U3+Umin;
dats.ud = Ud+Umin;
paraml  = [beta rhol rhoh psil psih ggl ggh gdl gdh sxl sgl sdl spl coefxl coefxh coefc gam];
paramh  = [beta rhol rhoh psil psih ggl ggh gdl gdh sxh sgh sdh sph coefxl coefxh coefc gam];
param   = [beta rhol rhoh psil psih ggl ggh gdl gdh sxl sxh sgl sgh sdl sdh spl sph coefxl coefxh coefc coefe accprob gam];
