clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
beta        = 0.975;
rhoh        = 0.85;
rhol        = 0.3;
psi         = 3;
gam         = 2.5;
alpha       = 0;

ncv0        = 2;
ncv         = 2;                    % Order of approximation C: V
lfile    = sprintf('decrules_last/resbayes_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*0.975);
fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_paramb(par1,par2);
sfile    = sprintf('decrules_last/resbayes_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
init_   = 1;
optim_  = 1;
strd    = [];
strp    = [];
%% Initial conditions
if init_
    thvl01  = 1.5;
    thvh01  = 1.5;
    thv01   = 1.5;
    thrl01  = 0.05;
    thrh01  = 0.05;
    thr01   = 0.05;
    thpl01  = 0.05;
    thph01  = 0.05;
    thp01   = 0.05;
    
    thvl0   = zeros(size(dats.tv,2),1);
    thvl0(1)= thvl01;
    thpl0   = zeros(size(dats.tp,2),1);
    thpl0(1)= thrl01;
    
    thvh0   = zeros(size(dats.tv,2),1);
    thvh0(1)= thvh01;
    thph0   = zeros(size(dats.tp,2),1);
    thph0(1)= thrh01;
    
    thv0    = zeros(size(data.tv3,2),1);
    thv0(1) = thv01;
    thv0(2) = 0.1;
    thp0    = zeros(size(data.tp3,2),1);
    thp0(1) = thr01;
else
    load(lfile)
    tmp     = dats.tv0*thvl;
    thvl0   = dats.tv\tmp;
    tmp     = dats.tv0*thvh;
    thvh0   = dats.tv\tmp;
    tmp     = data.tv40*thv;
    thv0    = data.tv4\tmp;
    
    tmp     = dats.tp0*thpl;
    thpl0   = dats.tp\tmp;
    tmp     = dats.tp0*thph;
    thph0   = dats.tp\tmp;
    tmp     = data.tp40*thp;
    thp0    = data.tp4\tmp;
end

paraml  = [alpha parl];
paramh  = [alpha parh];
param   = [alpha par];

%% Solve for the low rho case
thpl    = fcsolve(@residl_rp,thpl0,options,paraml,dats);
%% Solve for the high rho case
thph    = fcsolve(@residh_rp,thph0,options,paramh,dats);
%% Main model
data.thpl= thpl;
data.thph= thph;
thp     = fcsolve(@resid_rp,thp0,options,param,data);
%% Book keeping
save(sfile,'thpl','thph','thp');
%%
%% Moments+Accuracy Tables
RRfp    = comp_rrfp(param,data,thp);
strd    = char(strd,sprintf('& %5.2f & %5.2f & %5.2f & %5.2f & %5.2f && %5.2f & %5.2f & %5.2f \\\\',gam,alpha,RRfp.erf,RRfp.er,RRfp.eep,RRfp.srf,RRfp.sr,RRfp.sep));
pd      = comput_pdrbayes(param,data,thp);
pd1     = pd.pd1;
data    = pd.data;
y       = pd1;
ac      = autocorr(log(y),2);
X       = [gam alpha mean(y) std(y) mean(log(y)) std(log(y)) ac.ac(1) ac.ac(2)];
tmp     = sprintf('& %4.2f & %4.2f & %4.2f & %4.2f && %4.2f & %4.2f & %4.2f & %4.2f \\\\',X);
strp    = char(strp,tmp);
%%
disp('Return (Dividend Claim)')
disp(strd)
disp(' ')
disp('Price-dividend ratio')
disp(strp)
