function res = comp_rrfp(param,data,thp)
% risky assets are computed using asset prices
alpha   = param(1);
beta    = param(2);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
spl     = param(17);
sph     = param(18);
coefxl  = param(19);
coefxh  = param(20);
coefc   = param(21);
coefe   = param(22);
accprob = param(23);
gam     = param(24);

spi     = sqrt(pi);
spi3    = spi*spi*spi;


thpl    = data.thpl;
thph    = data.thph;
XX      = data.xd;

y       = data.y1d;
w       = data.w1d;
int3    = data.y3d;
wint3   = data.w3d;
kh      = data.kh;
kl      = data.kl;
fh      = data.fh;
fl      = data.fl;
nint3   = size(int3,1);
nint    = size(w,1);

c       = XX(:,1);
xl      = XX(:,2);
xh      = XX(:,3);
e       = XX(:,4);
Tc      = size(c,1);
HP      = hpmat(Tc,6.25);
hpc     = c-HP\c;


outl    = 0;
outh    = 0;
prh     = 0;
pr2h    = 0;
prl     = 0;
pr2l    = 0;
for i=1:nint;
    
    yi      = y(i);
    wi      = w(i);
    tmprh   = 0;
    tmprh2  = 0;
    
    tmprl   = 0;
    tmprl2  = 0;

    for j=1:nint3;
        gpl     = ggl+rhol*(xl+spl*yi)+sgl*int3(j,2)+sxl*int3(j,1);
        gph     = ggh+rhoh*(xh+sph*yi)+sgh*int3(j,2)+sxh*int3(j,1);
        gdpl    = gdl+psil*rhol*(xl+spl*yi)+sdl*int3(j,3)+psil*sxl*int3(j,1);
        gdph    = gdh+psih*rhoh*(xh+sph*yi)+sdh*int3(j,3)+psih*sxh*int3(j,1);
        %
        % nupij: i-> subscript; j->superscript
        %
        nupll   = [gpl-ggl-rhol*xl gdpl-gdl-psil*rhol*xl];
        nuphh   = [gph-ggh-rhoh*xh gdph-gdh-psih*rhoh*xh];
        nuphl   = [gpl-ggh-rhoh*xh gdpl-gdh-psih*rhoh*xh];
        nuplh   = [gph-ggl-rhol*xl gdph-gdl-psil*rhol*xl];
        %
        % compute the etas
        %
        l1      = likelihood(nupll,fl);
        l2      = likelihood(nuphl,fh);
        etapl   = e.*l1./(e.*l1+(1-e).*l2);
        
        l1      = likelihood(nuplh,fl);
        l2      = likelihood(nuphh,fh);
        etaph   = e.*l1./(e.*l1+(1-e).*l2);
        
        zxll    = coefxl*(rhol*xl+nupll*kl');
        zxlh    = coefxl*(rhol*xl+nuplh*kl');
        
        zxhh    = coefxh*(rhoh*xh+nuphh*kh');
        zxhl    = coefxh*(rhoh*xh+nuphl*kh');
                
        zel     = coefe*norm_inv(etapl);
        zeh     = coefe*norm_inv(etaph);

        zch      = coefc*(c+gph);
        zcl      = coefc*(c+gpl);
        
        Tpl     = makepoly3_pb([zxll,zxhl,zel]);
        Tpl2    = makepoly2_pb([zxll,zxhl]);

        Tph     = makepoly3_pb([zxlh,zxhh,zeh]);
        Tph2    = makepoly2_pb([zxlh,zxhh]);
        
        
        prpl                     = exp(Tpl*thp);
        prpl(etapl<accprob)      = exp(Tpl2(etapl<accprob,:)*thph);
        prpl(etapl>(1-accprob))  = exp(Tpl2(etapl>(1-accprob),:)*thpl);
        prpl                     = prpl.*exp(gdpl);
        
        prph                     = exp(Tph*thp);
        prph(etaph<accprob)      = exp(Tph2(etaph<accprob,:)*thph);
        prph(etaph>(1-accprob))  = exp(Tph2(etaph>(1-accprob),:)*thpl);
        prph                     = prph.*exp(gdph);
       
        tmprh   = tmprh+prph*wint3(j)/spi3;
        tmprh2  = tmprh2+(prph.*prph)*wint3(j)/spi3;
        
        tmprl    = tmprl+prpl*wint3(j)/spi3;
        tmprl2   = tmprl2+(prpl.*prpl)*wint3(j)/spi3;
    end
    tmpul   = exp(-gam*(ggl+rhol*(xl+spl*yi))+gam*gam*(sgl*sgl+sxl*sxl)/2);
    tmpuh   = exp(-gam*(ggh+rhoh*(xh+sph*yi))+gam*gam*(sgh*sgh+sxh*sxh)/2);
    
    prh      = prh+tmprh*wi/spi;
    pr2h     = pr2h+tmprh2*wi/spi;
    prl      = prl+tmprl*wi/spi;
    pr2l     = pr2l+tmprl2*wi/spi;

    outl    = outl+tmpul*wi/spi;
    outh    = outh+tmpuh*wi/spi;
    
end
pf      = beta*(e.*outl+(1-e).*outh);
rf      = 1./pf;
er      = e.*prl+(1-e).*prh;
er2     = e.*pr2l+(1-e).*pr2h;

s2r     = er2-er.*er;
ep      = er-rf;
res.rf  = rf;
res.r   = er;
res.r2  = er2;
res.ep  = ep;
res.erf = 100*mean(rf-1);
res.er  = 100*mean(er-1);
res.eep = 100*mean(ep);
res.hpc = hpc;
res.srf = 100*std(rf);
        % if 0 then mean of square root
met_=0; %
        % if 1 then square root of mean

if met_
    tmp1    = mean(s2r);
    tmp2    = mean((rf-mean(rf)).*(rf-mean(rf)));
    tmp3    = mean((er-mean(er)).*(rf-mean(rf)));
    res.sep = 100*sqrt(tmp1+tmp2-2*tmp3);
    res.sr  = 100*sqrt(mean(s2r));
else
    tmp1    = s2r;
    tmp2    = (rf-mean(rf)).*(rf-mean(rf));
    tmp3    = (er-mean(er)).*(rf-mean(rf));
    res.sep = 100*mean(sqrt(tmp1+tmp2-2*tmp3));
    res.sr  = 100*mean(sqrt(s2r));
end

tmp     = corrcoef(rf,hpc);
res.crfc= tmp(1,2);
tmp     = corrcoef(er,hpc);
res.cerc= tmp(1,2);
tmp     = corrcoef(ep,hpc);
res.cepc= tmp(1,2);

