clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
optmin  = optimset('maxfunevals',1e9,'maxiter',1e9,'display','iter','tolfun',1e-6,'tolx',1e-6);

%% Parameters of approximation
ncv     = 3;						% Order of approximation C: V
nxv     = 2;						% Order of approximation X_hat: V

ncr     = 3;						% Order of approximation C: R
nxr     = 3;						% Order of approximation X_hat: R

ncp     = 2;						% Order of approximation C: P
nxp     = 5;						% Order of approximation X_hat: P

nca     = 5;						% Order of approximation C: P
nxa     = 5;						% Order of approximation X_hat: P

nodesx	= 8;						% Number of nodes for X_hat
nodesc	= 8;						% Number of nodes for C

%% Main loop
%% Structural parameters of the model
beta        = 0.975;
bench       = 1.5;
rho         = 0.85;
psi         = 3;
gam         = 2.5;
%% Find an initial value for Thv, Thr, Thp
a0          = 7.242;
init_       = 1;
thv01       = 1.5;
thr01       = 0.05;
thp01       = 0.05;
[dat,par]   = build_params([gam rho psi beta],[ncv nxv ncr nxr ncp nxp nca nxa nodesc nodesx]);

switch init_
    case 1
        thv0    = zeros(size(dat.tv,2),1);
        thv0(1) = thv01;
        thv0=[    1.5971
            0.0234
            -0.0003
            0.1077
            -0.0035
            0.0000
            -0.0101
            0.0003
            0.0008];
        thv=thv0;
        save temp.mat thv
        save temp2.mat thv
        thp0    = zeros(size(dat.tp,2),1);
        thp0(1) = thp01;
    otherwise
        load(sprintf('tmp%d.mat',100*rho));
        thv0=thv;
        thp0=thp;
end

param       = [0 par];
alpha       = fminsearch(@fcalib_alpha,a0,optmin,param,dat,bench);
[tmp,thv0]  = fcalib_alpha(alpha,param,dat,bench);
%%
param       = [alpha par];
% Value Function
thv         = fcsolve(@resid_v,thv0,options,param,dat);


Cd      = dat.xd(:,1);
Xd      = dat.xd(:,2);
gbar    = param(6);
gdbar   = param(7);
sx      = param(8);
sg      = param(9);
sd      = param(10);
sp      = param(11);

xmax    = 0.2;
xmin    =-0.2;

E0=[rho*min(Xd)+gbar sqrt(sx*sx+sg*sg);
    rho*mean(Xd)+gbar sqrt(sx*sx+sg*sg);
    rho*max(Xd)+gbar sqrt(sx*sx+sg*sg)];
%%
nbx     = 1001;
x       = linspace(xmin,xmax,nbx)';
dx      = diff(x);
mx      = (x(2:nbx)+x(1:nbx-1))/2;
P       = sp*sp;
T       = length(Xd);
Exs     = zeros(T,1);
Vxs     = zeros(T,1);
Exs2    = zeros(T,1);
Vxs2    = zeros(T,1);
for t=1:T
    dat.xcx     = [Cd(t) Xd(t)];
    dat.x       = x;
    twist       = ftwist(thv,param,dat);
    ftmp        = exp(-0.5*(x-Xd(t)).*(x-Xd(t))/P)/sqrt(2*pi*P);
    ftmp        = ftmp/sum(dx.*(ftmp(2:nbx)+ftmp(1:nbx-1))/2);
    f           = dx.*(ftmp(2:nbx)+ftmp(1:nbx-1))/2;
    fstmp       = ftmp.*twist;
    fstmp       = fstmp/sum(dx.*(fstmp(2:nbx)+fstmp(1:nbx-1))/2);
    fs          = dx.*(fstmp(2:nbx)+fstmp(1:nbx-1))/2;
    
    Exs(t)  =  mx'*fs;
    Vxs(t)  = (mx.*mx)'*fs-Exs(t)*Exs(t);
    res     = comp_momx(thv,param,dat);
    Exs2(t) = res.etx;
    Vxs2(t) = res.vtx;
end
% The series Vxs2 is used in the file replic_figure5a as measure of volatility