clear all;
% clc
%% Path + basic parameters
addpath ../method
addpath ../data_last
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
optmin  = optimset('maxfunevals',1e9,'maxiter',1e9,'display','iter','tolfun',1e-6,'tolx',1e-6);

%% Parameters of approximation
ncv     = 3;						% Order of approximation C: V
nxv     = 2;						% Order of apprmoximation X_hat: V

ncr     = 3;						% Order of approximation C: R
nxr     = 3;						% Order of approximation X_hat: R

ncp     = 3;						% Order of approximation C: P
nxp     = 3;						% Order of approximation X_hat: P

nca     = 5;						% Order of approximation C: P
nxa     = 5;						% Order of approximation X_hat: P

nodesx	= 8;						% Number of nodes for X_hat
nodesc	= 8;						% Number of nodes for C

%% Structural parameters of the model
BET         = 0.975;
bench       = 1.5;
rho         = 0.85;
psi         = 3;
fid         = 1;
ainit       = 40;
for ibet=1:length(BET)
    beta        = BET(ibet);
    init_       = 1;
    a0          = ainit;
    GAM         = [1.001 1.5:0.5:2.5];
    ng          = length(GAM);
    thv01       = 0.35;
    thr01       = 0.05;
    thp01       = 0.05;
    [dat,par]   = build_params([1 rho psi beta],[ncv nxv ncr nxr ncp nxp nca nxa nodesc nodesx]);
    
    switch init_
        case 1
            thv0    = zeros(size(dat.tv,2),1);
            thv0(1) = thv01;
            thv=thv0;
            save temp.mat thv
            save temp2.mat thv
            thr0    = zeros(size(dat.tr,2),1);
            thr0(1) = thr01;
            thp0    = zeros(size(dat.tp,2),1);
            thp0(1) = thp01;
        otherwise
            load(sprintf('tmp%d.mat',100*rho));
            thv0=thv;
            thr0=thr;
            thp0=thp;
    end
    alpha       = zeros(ng,1);
    str_acc     = [];
    str_con     = [];
    str_div     = [];
    str_cpd     = [];
    str_cxd     = [];
    str_pd      = [];
    for i=1:ng
        tstart      = tic;
        gam         = GAM(i);
        [dat,par]   = build_params([gam rho psi beta],[ncv nxv ncr nxr ncp nxp nca nxa nodesc nodesx]);
        
        param       = [0 par];
        alpha(i)    = fminsearch(@fcalib_alpha,a0,optmin,param,dat,bench);
        [tmp,thv0]  = fcalib_alpha(alpha(i),param,dat,bench);
        a0          = alpha(i);
        if i==1
            ainit   = a0;
        end
        param(1)    = alpha(i);
        str1        = sprintf('gamma = %6.4f, alpha = %6.4f',gam,alpha(i));
        %% Value Function
        thv         = fcsolve(@resid_v,thv0,options,param,dat);
        thv0        = thv;
        %% Dividend claim
        thp         = fcsolve(@resid_rp,thp0,options,param,dat,thv);
        thp0        = thp;
        %% Bookkeeping
        save temp.mat thv thp;
        %     save temp2.mat thv thr thp;
        %% Moments
        RRfp        = comp_rrfp(param,dat,thv,thp);     % generates D based returns        
        str_div     = char(str_div,sprintf('%5.2f & %5.2f & %5.2f & %5.2f && %5.2f & %5.2f & %5.2f \\\\',gam,alpha(i),RRfp.er,RRfp.eep,RRfp.srf,RRfp.sr,RRfp.sep));
        tend        = toc(tstart);
        str2        = sprintf('\t Solved in %g sec.',tend);
        disp(strcat(str1,str2))
    end
    %%
    fprintf(fid,'beta=%5.3f, psi=%4.2f, rho=%4.2f\n\n',beta,psi,rho);
    fprintf(fid,' ');
    fprintf(fid,'Returns (Dividend Claims)');
    for i=1:size(str_div,1);
        fprintf(fid,'%s\n',str_div(i,:));
    end
end
% Note: the last realization of RRfp.ep is used in the replic_figure 5a
% as input for the E_t(R_{t+1})-R^f_t
%%
xx=comp_momx(thv,param,dat)