clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
optmin  = optimset('maxfunevals',1e9,'maxiter',1e9,'display','iter','tolfun',1e-6,'tolx',1e-6);

%% Parameters of approximation
ncv     = 3;						% Order of approximation C: V
nxv     = 2;						% Order of approximation X_hat: V

ncr     = 3;						% Order of approximation C: R
nxr     = 3;						% Order of approximation X_hat: R

ncp     = 2;						% Order of approximation C: P
nxp     = 5;						% Order of approximation X_hat: P

nca     = 5;						% Order of approximation C: P
nxa     = 5;						% Order of approximation X_hat: P

nodesx	= 8;						% Number of nodes for X_hat
nodesc	= 8;						% Number of nodes for C

%% Main loop
%% Structural parameters of the model
beta        = 0.975;
bench       = 1.5;
rho         = 0.85;
psi         = 3;
gam         = 2.5;
%% Find an initial value for Thv, Thr, Thp
a0          = 7.242;
init_       = 1;
thv01       = 1.5;
thr01       = 0.05;
thp01       = 0.05;
[dat,par]   = build_params([gam rho psi beta],[ncv nxv ncr nxr ncp nxp nca nxa nodesc nodesx]);

switch init_
    case 1
        thv0    = zeros(size(dat.tv,2),1);
        thv0(1) = thv01;
        thv0=[    1.5971
    0.0234
   -0.0003
    0.1077
   -0.0035
    0.0000
   -0.0101
    0.0003
    0.0008];
        thv=thv0;
        save temp.mat thv
        save temp2.mat thv
        thp0    = zeros(size(dat.tp,2),1);
        thp0(1) = thp01;
    otherwise
        load(sprintf('tmp%d.mat',100*rho));
        thv0=thv;
        thp0=thp;
end

param       = [0 par];
alpha       = fminsearch(@fcalib_alpha,a0,optmin,param,dat,bench);
[tmp,thv0]  = fcalib_alpha(alpha,param,dat,bench);
%%
param       = [alpha par];
% Value Function
thv         = fcsolve(@resid_v,thv0,options,param,dat);


Cd      = dat.xd(:,1);
Xd      = dat.xd(:,2);
gbar    = param(6);
gdbar   = param(7);
sx      = param(8);
sg      = param(9);
sd      = param(10);
sp      = param(11);

xmax    = 0.2;
xmin    =-0.2;

E0=[rho*min(Xd)+gbar sqrt(sx*sx+sg*sg);
    rho*mean(Xd)+gbar sqrt(sx*sx+sg*sg);
    rho*max(Xd)+gbar sqrt(sx*sx+sg*sg)];

nbx     = 1001;
x       = linspace(xmin,xmax,nbx)';
dx      = diff(x);
mx      = (x(2:nbx)+x(1:nbx-1))/2;
g       = linspace(-0.2,0.2,nbx)';
dg      = diff(g);
mg      = (g(2:nbx)+g(1:nbx-1))/2;
d       = linspace(-0.6,0.6,nbx)';
dd      = diff(d);
md      = (d(2:nbx)+d(1:nbx-1))/2;
FG      = zeros(nbx-1,3);
FGs     = zeros(nbx-1,3);
FR      = zeros(nbx-1,3);
FD      = zeros(nbx-1,3);
FDs     = zeros(nbx-1,3);
FRd     = zeros(nbx-1,3);
f       = zeros(nbx-1,3);
twist   = zeros(nbx-1,3);
fs      = zeros(nbx-1,3);
fg      = zeros(nbx-1,3);
mc      = mean(Cd);
xx      = mean(Xd);
P       = sp*sp;
for t=1:length(xx);
    dat.xcx     = [mc xx(t)];
    dat.x       = x;
    twist       = ftwist(thv,param,dat);
    ftmp        = exp(-0.5*(x-xx(t)).*(x-xx(t))/P)/sqrt(2*pi*P);
    ftmp        = ftmp/sum(dx.*(ftmp(2:nbx)+ftmp(1:nbx-1))/2);
    f(:,t)      = dx.*(ftmp(2:nbx)+ftmp(1:nbx-1))/2;
    fstmp       = ftmp.*twist;
    fstmp       = fstmp/sum(dx.*(fstmp(2:nbx)+fstmp(1:nbx-1))/2);
    fs(:,t)     = dx.*(fstmp(2:nbx)+fstmp(1:nbx-1))/2;
    fg          = zeros(nbx-1,nbx-1);
    fd          = zeros(nbx-1,nbx-1);
    for i=1:nbx-1
        gtmp    = exp(-0.5*(g-gbar-rho*mx(i)).*(g-gbar-rho*mx(i))/(sx*sx+sg*sg))/sqrt(2*pi*(sx*sx+sg*sg));
        gtmp    = gtmp/sum(dg.*(gtmp(2:nbx)+gtmp(1:nbx-1))/2);
        fg(:,i) = dg.*(gtmp(2:nbx)+gtmp(1:nbx-1))/2;
        
        dtmp    = exp(-0.5*(d-gdbar-psi*rho*mx(i)).*(d-gdbar-psi*rho*mx(i))/(psi*psi*sx*sx+sd*sd))/sqrt(2*pi*(psi*psi*sx*sx+sd*sd));
        dtmp    = dtmp/sum(dd.*(dtmp(2:nbx)+dtmp(1:nbx-1))/2);
        fd(:,i) = dd.*(dtmp(2:nbx)+dtmp(1:nbx-1))/2;
    end;    
    FG(:,t)     = fg*f(:,t);
    FGs(:,t)    = fg*fs(:,t);
    FD(:,t)     = fd*f(:,t);
    FDs(:,t)    = fd*fs(:,t);
    rtmp        = exp(-0.5*(g-gbar-rho*xx(t)).*(g-gbar-rho*xx(t))/(sx*sx+sg*sg))/sqrt(2*pi*(sx*sx+sg*sg));
    rtmp        = rtmp/sum(dg.*(rtmp(2:nbx)+rtmp(1:nbx-1))/2);
    FR(:,t)     = dg.*(rtmp(2:nbx)+rtmp(1:nbx-1))/2;
    
    rdtmp       = exp(-0.5*(d-gdbar-rho*psi*xx(t)).*(d-gdbar-rho*psi*xx(t))/(psi*psi*sx*sx+sd*sd))/sqrt(2*pi*(psi*psi*sx*sx+sd*sd));
    rdtmp       = rdtmp/sum(dd.*(rdtmp(2:nbx)+rdtmp(1:nbx-1))/2);
    FRd(:,t)    = dd.*(rdtmp(2:nbx)+rdtmp(1:nbx-1))/2;
    
end
%%
Ex  = mx'*f(:,1);
Sx  = sqrt((mx.*mx)'*f(:,1)-Ex*Ex);
E2x = ((mx-Ex).*(mx-Ex))'*f(:,1);
E3x = ((mx-Ex).*(mx-Ex).*(mx-Ex))'*f(:,1);
E4x = ((mx-Ex).*(mx-Ex).*(mx-Ex).*(mx-Ex))'*f(:,1);
Skx = E3x/(Sx^3);
Kux = E4x/(E2x*E2x)-3;

Exs = mx'*fs(:,1);
Sxs = sqrt((mx.*mx)'*fs(:,1)-Exs*Exs);
E2xs= ((mx-Exs).*(mx-Exs))'*fs(:,1);
E3xs= ((mx-Exs).*(mx-Exs).*(mx-Exs))'*fs(:,1);
E4xs= ((mx-Exs).*(mx-Exs).*(mx-Exs).*(mx-Exs))'*fs(:,1);
Skxs= E3xs/(Sxs^3);
Kuxs= E4xs/(E2xs*E2xs)-3;

fprintf('%7.4f & %7.4f \n',Ex,Sx)
fprintf('%7.4f & %7.4f \n',Exs,Sxs)
fprintf('%7.4f & %7.4f \n',Skx,Kux)
fprintf('%7.4f & %7.4f \n',Skxs,Kuxs)
%%
Er  = mg'*FR(:,1);
Sr  = sqrt((mg.*mg)'*FR(:,1)-Er*Er);
E2r = ((mg-Er).*(mg-Er))'*FR(:,1);
E3r = ((mg-Er).*(mg-Er).*(mg-Er))'*FR(:,1);
E4r = ((mg-Er).*(mg-Er).*(mg-Er).*(mg-Er))'*FR(:,1);
Skr = E3r/(Sr^3);
Kur = E4r/(E2r*E2r)-3;
Eg  = mg'*FG(:,1);
Sg  = sqrt((mg.*mg)'*FG(:,1)-Eg*Eg);
E2g = ((mg-Eg).*(mg-Eg))'*FG(:,1);
E3g = ((mg-Eg).*(mg-Eg).*(mg-Eg))'*FG(:,1);
E4g = ((mg-Eg).*(mg-Eg).*(mg-Eg).*(mg-Eg))'*FG(:,1);
Skg = E3g/(Sg^3);
Kug = E4g/(E2g*E2g)-3;
Es  = mg'*FGs(:,1);
Ss  = sqrt((mg.*mg)'*FGs(:,1)-Es*Es);
E2s = ((mg-Es).*(mg-Es))'*FGs(:,1);
E3s = ((mg-Es).*(mg-Es).*(mg-Es))'*FGs(:,1);
E4s = ((mg-Es).*(mg-Es).*(mg-Es).*(mg-Es))'*FGs(:,1);
Sks = E3s/(Ss^3);
Kus = E4s/(E2s*E2s)-3;

fprintf('%7.4f & %7.4f \n',Er,Sr)
fprintf('%7.4f & %7.4f \n',Eg,Sg)
fprintf('%7.4f & %7.4f \n',Es,Ss)
fprintf('%7.4f & %7.4f \n',Skr,Kur)
fprintf('%7.4f & %7.4f \n',Skg,Kug)
fprintf('%7.4f & %7.4f \n',Sks,Kus)
[Er Sr Skr Kur;Eg Sg Skg Kug;Es Ss Sks Kus]
%%
Erd = md'*FRd(:,1);
Srd = sqrt((md.*md)'*FRd(:,1)-Erd*Erd);
E2rd= ((md-Erd).*(md-Erd))'*FRd(:,1);
E3rd= ((md-Erd).*(md-Erd).*(md-Erd))'*FRd(:,1);
E4rd= ((md-Erd).*(md-Erd).*(md-Erd).*(md-Erd))'*FRd(:,1);
Skrd= E3rd/(Srd^3);
Kurd= E4rd/(E2rd*E2rd)-3;

Egd = md'*FD(:,1);
Sgd = sqrt((md.*md)'*FD(:,1)-Egd*Egd);
E2gd= ((md-Egd).*(md-Egd))'*FD(:,1);
E3gd= ((md-Egd).*(md-Egd).*(md-Egd))'*FD(:,1);
E4gd= ((md-Egd).*(md-Egd).*(md-Egd).*(md-Egd))'*FD(:,1);
Skgd= E3gd/(Sgd^3);
Kugd= E4gd/(E2gd*E2gd)-3;

Esd = md'*FDs(:,1);
Ssd = sqrt((md.*md)'*FDs(:,1)-Esd*Esd);
E2sd= ((md-Esd).*(md-Esd))'*FDs(:,1);
E3sd= ((md-Esd).*(md-Esd).*(md-Esd))'*FDs(:,1);
E4sd= ((md-Esd).*(md-Esd).*(md-Esd).*(md-Esd))'*FDs(:,1);
Sksd= E3sd/(Ssd^3);
Kusd= E4sd/(E2sd*E2sd)-3;

fprintf('%7.4f & %7.4f \n',Erd,Srd)
fprintf('%7.4f & %7.4f \n',Egd,Sgd)
fprintf('%7.4f & %7.4f \n',Esd,Ssd)

fprintf('%7.4f & %7.4f \n',Skrd,Kurd)
fprintf('%7.4f & %7.4f \n',Skgd,Kugd)
fprintf('%7.4f & %7.4f \n',Sksd,Kusd)

FR=FR(:,1);
FB=FG(:,1);
FT=FGs(:,1);
save data_distg_1rho.mat mg FR FB FT
%%
fsz=16;
h=plot(mg,[FR(:,1) FG(:,1) FGs(:,1)]);
set(h(1),'color',0.5*ones(3,1),'linewidth',2,'linestyle','-');
set(h(2),'color','k','linewidth',2,'linestyle','--');
set(h(3),'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fsz,'ytick',[]);
xlabel('g','fontname','times','fontsize',fsz);
legend('R.E.','Bayesians','Twisted','Location','NorthWest')

pause;
printpdf(sprintf('distrib1rho_%d',100*rho))
print(sprintf('distrib1rho_%d.eps',100*rho),'-depsc2');
close
