function [data,param]   = build_params(param1,param2)

gam     = param1(1);
rho     = param1(2);
psi     = param1(3);
beta    = param1(4);
ncv     = param2(1);
nxv     = param2(2);
ncr     = param2(3);
nxr     = param2(4);
ncp     = param2(5);
nxp     = param2(6);
nca     = param2(7);
nxa     = param2(8);
nodesc  = param2(9);
nodesx  = param2(10);

%
% Prepare functions that compute polynomials
%
folder='sol_functions/';
write_poly('makepoly_v',[ncv,nxv],folder); % Function for V
write_poly('makepoly_r',[ncr,nxr],folder); % Function for R
write_poly('makepoly_p',[ncp,nxp],folder); % Function for P
write_poly('makepoly_a',[nca,nxa],folder); % Function for AP

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%    Algorithm Parameters + Preparation of polynomial and Integration     %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[zx,wx] = gauss_herm(nodesx);       % Interpolation nodes
[zc,wc] = gauss_herm(nodesc);       % Interpolation nodes

Zmatrix = [kron(zc,ones(nodesx,1)),kron(ones(nodesc,1),zx)];
data.tv = makepoly_v(Zmatrix);      % Polynomial for V
data.tr = makepoly_r(Zmatrix);      % Polynomial for R
data.tp = makepoly_p(Zmatrix);      % Polynomial for P
data.ta = makepoly_a(Zmatrix);      % Polynomial for P

data.wx = kron(wc,wx);              % Integration for residuals
integ   = integ_data(1);            % Integration wrt shocks
% If parameter=1 -> Mono
% If parameter=0 -> Gauss-Hermite
data.y1d= integ.y1d;
data.w1d= integ.w1d;
data.y3d= integ.y3d;
data.w3d= integ.w3d;
data.ye = integ.ye;

coefx   = 40;                       % Scaling coefficient for X_hat
tmp     = gauss_herm(4);
coefc   = zc(1)/tmp(1);             % Scaling coefficient for C_hat;

X       = zx/coefx;                 % Builds X_hat
C       = zc/coefc;                 % Builds C

data.x  = [kron(C,ones(nodesx,1)) kron(ones(nodesc,1),X)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%            Historical Data + Invariant Structural Parameters            %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gbar    = 0.0192520;                % Average consumption growth
gdbar   = 0.0231930;                % Average dividend growth
if psi==3
    if rho==0.85
        data_psi300_rhol30_rhoh85;
        sg      = sqrt(0.00025898);
        sd      = sqrt(0.04755800);
        sx      = sqrt(0.00051808);
    else
        data_psi300_rhol30_rhoh90;
        sg      = sqrt(0.00019836);
        sd      = sqrt(0.04730800);
        sx      = sqrt(0.00059686);
    end
else
    if rho==0.85
        data_psi250_rhol30_rhoh85;
        sg      = sqrt(0.00026453);
        sd      = sqrt(0.04808400);
        sx      = sqrt(0.00050760);
    else
        data_psi250_rhol30_rhoh90;
        sg      = sqrt(0.00020334);
        sd      = sqrt(0.04772500);
        sx      = sqrt(0.00058660);
    end
end

whole   = 1930:2011;
t0      = 1977;
t1      = 2011;
smpl    = find(whole==t0):find(whole==t1);
Xd      = DATX(smpl,3);
Cd      = log(DATX(smpl,5)/10000);

data.xpd = [Cd Xd];
data.tppd= makepoly_p([Cd*coefc Xd*coefx]);      % Polynomial for P

data.xd = [Cd(2:end) Xd(2:end)];
Zmatrix = [Cd(2:end)*coefc Xd(2:end)*coefx];
data.tvd= makepoly_v(Zmatrix);      % Polynomial for V
data.trd= makepoly_r(Zmatrix);      % Polynomial for R
data.tpd= makepoly_p(Zmatrix);      % Polynomial for P
data.tad= makepoly_a(Zmatrix);      % Polynomial for P
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                            Kalman Filter                                %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
crit    = 1;
tol     = 1e-12;
P0      = 1;
while crit>tol
    F   = [P0+sg*sg psi*P0;psi*P0 psi*psi*P0+sd*sd];
    IF  = chol(F);
    IF  = IF\eye(2);
    IF  = IF*IF';
    P   = rho*rho*P0-rho*rho*P0*P0*([1 psi]*IF*[1 psi]')+sx*sx;
    crit= abs(P-P0);
    P0  = P;
end
sp      = sqrt(P);
data.k  = rho*P*([1 psi]*IF);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
%                                Utility                                  %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

U       = exp((1-gam)*data.x(:,1))/(1-gam);
Ud      = exp((1-gam)*data.xd(:,1))/(1-gam);
Umin    = 0;
if gam>1;
    CM      = 1.01*data.x(1,1);
    Umin    = abs(exp((1-gam)*CM)/(1-gam));
else
    CM      = 1.01*data.x(1,1);
    Umin    =-abs(exp((1-gam)*CM)/(1-gam));
end
data.umin= Umin;
data.u  = (U+Umin);
data.ud = (Ud+Umin);

param   = [beta gam rho psi gbar gdbar sx sg sd sp coefx coefc ncv nxv ncr nxr ncp nxp nca nxa];
