clear all;
% function graph_comp_stat_alpha
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
beta        = 0.975;
rhoh        = 0.85;
rhol        = 0.3;
psi         = 3;
gam         = 2.5;
alpha       = 11.35;
almin       = 7;
almax       = 15;
XX          = [linspace(11.35,almin,5)' linspace(11.35,almax,5)'];
nx          = size(XX,1);
ncv0        = 3;
ncv         = 3;                    % Order of approximation C: V
lfile    = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);

par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);
%% loop
Erf     = zeros(nx,2);    
Er      = zeros(nx,2);    
Eep     = zeros(nx,2);    
Epd     = zeros(nx,2);    
Elpd    = zeros(nx,2);    
Srf     = zeros(nx,2);    
Sr      = zeros(nx,2);    
Sep     = zeros(nx,2);    
Spd     = zeros(nx,2);    
Slpd    = zeros(nx,2);    
Epd0        = zeros(nx,2);
Epd1        = zeros(nx,2);
Eepd0       = zeros(nx,2);
Eepd1       = zeros(nx,2);
Eepd        = zeros(nx,2);
for j=1:2;
    load(lfile)
    tmp     = dats.tv0*thvl;
    thvl0   = dats.tv\tmp;
    tmp     = dats.tv0*thvh;
    thvh0   = dats.tv\tmp;
    tmp     = data.tv40*thv;
    thv0    = data.tv4\tmp;
    
    tmp     = dats.tp0*thpl;
    thpl0   = dats.tp\tmp;
    tmp     = dats.tp0*thph;
    thph0   = dats.tp\tmp;
    tmp     = data.tp40*thp;
    thp0    = data.tp4\tmp;
    
    for i=1:nx
        fprintf('alpha=%g\n',XX(i,j))
        paraml  = [XX(i,j) parl];
        paramh  = [XX(i,j) parh];
        param   = [XX(i,j) par];
        
        %% Solve for the low rho case
        thvl    = fcsolve(@residl_v,thvl0,options,paraml,dats);
        thvl0   = thvl;
        thpl    = fcsolve(@residl_rp,thpl0,options,paraml,dats,thvl);
        thpl0   = thpl;
        %% Solve for the high rho case
        thvh    = fcsolve(@residh_v,thvh0,options,paramh,dats);
        thvh0   = thvh;
        thph    = fcsolve(@residh_rp,thph0,options,paramh,dats,thvh);
        thph0   = thph;
        %% Main model
        data.thvl= thvl;
        data.thvh= thvh;
        data.thpl= thpl;
        data.thph= thph;
        
        thv     = fcsolve(@resid_v,thv0,options,param,data);
        thv0    = thv;
        thp     = fcsolve(@resid_rp,thp0,options,param,data,thv);
        thp0    = thp;
        
        %%
        %% Moments+Accuracy Tables
        RRfp        = comp_rrfp(param,data,thv,thp);
        Rpd         = comput_pdr(param,data,thp);
        Erf(i,j)    = RRfp.erf;
        Er(i,j)     = RRfp.er;
        Eep(i,j)    = RRfp.eep;
        Epd0(i,j)   = mean(Rpd.pd0);
        Epd1(i,j)   = mean(Rpd.pd1);
        Eepd0(i,j)  = mean(Rpd.epd0);
        Eepd1(i,j)  = mean(Rpd.epd1);
        Eepd(i,j)   = mean(Rpd.epd);
        Srf(i,j)    = RRfp.srf;
        Sr(i,j)     = RRfp.sr;
        Sep(i,j)    = RRfp.sep;
    end
end
[XX,I]  = unique([flipud(XX(:,1));XX(:,2)]);
Erf     = [flipud(Erf(:,1));Erf(:,2)];
Erf     = Erf(I);
Er      = [flipud(Er(:,1));Er(:,2)];
Er      = Er(I);
Eep     = [flipud(Eep(:,1));Eep(:,2)];
Eep     = Eep(I);
Epd0    = [flipud(Epd0(:,1));Epd0(:,2)];
Epd0    = Epd0(I);
Epd1    = [flipud(Epd1(:,1));Epd1(:,2)];
Epd1    = Epd1(I);
Eepd0   = [flipud(Eepd0(:,1));Eepd0(:,2)];
Eepd0   = Eepd0(I);
Eepd1   = [flipud(Eepd1(:,1));Eepd1(:,2)];
Eepd1   = Eepd1(I);
Eepd    = [flipud(Eepd(:,1));Eepd(:,2)];
Eepd    = Eepd(I);
Srf     = [flipud(Srf(:,1));Srf(:,2)];
Srf     = Srf(I);
Sr      = [flipud(Sr(:,1));Sr(:,2)];
Sr      = Sr(I);
Sep     = [flipud(Sep(:,1));Sep(:,2)];
Sep     = Sep(I);
Spd     = [flipud(Spd(:,1));Spd(:,2)];
Spd     = Spd(I);
Slpd    = [flipud(Slpd(:,1));Slpd(:,2)];
Slpd    = Slpd(I);
save meanret_alpha_2rhos.mat XX Erf Er Eep Epd0 Epd1 Eepd0 Eepd1 Eepd;
%% Graphs
method  = 'pchip';
fs      = 12;
X       = linspace(almin,almax,50);
Y       = interp1(XX,Erf,X,method);
subplot(231);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Riskfree Rate','fontname','times','fontsize',fs);

Y       = interp1(XX,Er,X,method);
subplot(232);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Risky Rate','fontname','times','fontsize',fs);

Y       = interp1(XX,Eep,X,method);
subplot(233);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Equity Premium','fontname','times','fontsize',fs);

printpdf('cs_meanret_alpha2')
print -depsc2 cs_meanret_alpha2.eps

Y       = interp1(XX,Srf,X,method);
subplot(231);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Riskfree Rate','fontname','times','fontsize',fs);

Y       = interp1(XX,Sr,X,method);
subplot(232);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Risky Rate','fontname','times','fontsize',fs);

Y       = interp1(XX,Sep,X,method);
subplot(233);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Equity Premium','fontname','times','fontsize',fs);

printpdf('cs_stdret_alpha2')
print -depsc2 cs_stdret_alpha2.eps

Y       = interp1(XX,Epd,X,method);
subplot(221);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Price/Dividend Ratio','fontname','times','fontsize',fs);

Y       = interp1(XX,Elpd,X,method);
subplot(222);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Price/Dividend Ratio (Logs)','fontname','times','fontsize',fs);

printpdf('cs_meanpd_alpha2')
print -depsc2 cs_meanpd_alpha2.eps

Y       = interp1(XX,Spd,X,method);
subplot(221);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Price/Dividend Ratio','fontname','times','fontsize',fs);

Y       = interp1(XX,Slpd,X,method);
subplot(222);plot(X,Y,'color','k','linewidth',2);
set(gca,'fontname','times','fontsize',fs);
xlabel('\alpha','fontname','times','fontsize',fs);
title('Price/Dividend Ratio (Logs)','fontname','times','fontsize',fs);

printpdf('cs_stdpd_alpha2')
print -depsc2 cs_stdpd_alpha2.eps