% Computes the solution decision rules
clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
beta        = 0.965;0.975;
rhoh        = 0.85;
rhol        = 0.3;
psi         = 3;
gam         = 2.5;
alpha       = 5;11.35;

ncv0        = 3;                    % Order of approximation C: V former case
ncv         = 3;                    % Order of approximation C: V
lfile       = sprintf('decrules_last/resa0_%d_%d_%d_%d_%d',100*2.5,100*psi,100*rhoh,100*rhol,1000*beta);
% lfile       = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*2.5,100*psi,100*rhoh,100*rhol,960);
% lfile       = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
% lfile       = 'decrules_last/res_250_300_85_30_975';
fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);
sfile    = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
% sfile    = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
init_   = 1;  % set to 1 to start from scratch, 0 to load pre-existing results
optim_  = 1;  % set to 1 to solve
%% Initial conditions
if init_
    thvl01  = 2;
    thvh01  = 1.5;
    thv01   = 1.7;
    thrl01  = 0.05;
    thrh01  = 0.05;
    thr01   = 0.05;
    thpl01  = 0.05;
    thph01  = 0.05;
    thp01   = 0.05;

    thvl0   = zeros(size(dats.tv,2),1);
    thvl0(1)= thvl01;
    thrl0   = zeros(size(dats.tr,2),1);
    thrl0(1)= thr01;
    thpl0   = zeros(size(dats.tp,2),1);
    thpl0(1)= thrl01;
    
    thvh0   = zeros(size(dats.tv,2),1);
    thvh0(1)= thvh01;
    thrh0   = zeros(size(dats.tr,2),1);
    thrh0(1)= thr01;
    thph0   = zeros(size(dats.tp,2),1);
    thph0(1)= thrh01;
    
    thv0    = zeros(size(data.tv4,2),1);
    thv0(1) = thv01;
    thv0(2) = .01;
    thr0    = zeros(size(data.tr4,2),1);
    thr0(1) = thr01;
    thp0    = zeros(size(data.tp4,2),1);
    thp0(1) = thr01;
else
    load(lfile)
    tmp     = dats.tv0*thvl;
    thvl0   = dats.tv\tmp;
    tmp     = dats.tv0*thvh;
    thvh0   = dats.tv\tmp;
    tmp     = data.tv40*thv;
    thv0    = data.tv4\tmp;
        
    tmp     = dats.tp0*thpl;
    thpl0   = dats.tp\tmp;
    tmp     = dats.tp0*thph;
    thph0   = dats.tp\tmp;
    tmp     = data.tp40*thp;
    thp0    = data.tp4\tmp;
end

paraml  = [alpha parl];
paramh  = [alpha parh];
param   = [alpha par];

if optim_
    %% Solve for the low rho case
    thvl    = fcsolve(@residl_v,thvl0,options,paraml,dats);
    thpl    = fcsolve(@residl_rp,thpl0,options,paraml,dats,thvl);
    %% Solve for the high rho case
    thvh    = fcsolve(@residh_v,thvh0,options,paramh,dats);
    thph    = fcsolve(@residh_rp,thph0,options,paramh,dats,thvh);
    %% Main model
    data.thvl= thvl;
    data.thvh= thvh;
    data.thpl= thpl;
    data.thph= thph;
    

    thv     = fcsolve(@resid_v,thv0,options,param,data);
    thp     = fcsolve(@resid_rp,thp0,options,param,data,thv);
    %% Book keeping
    save(sfile,'thvl','thpl','thvh','thph','thv','thp');
    %%
else
    load(lfile)

    data.thvl= thvl;
    data.thvh= thvh;
    data.thpl= thpl;
    data.thph= thph;
    data.thv = thv;
    data.thp = thp;
end

