% Generates Table 2 for returns.
clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model

str = {'','','','','$\rho_h=0.90$','$\rho_l=0.25$',...
    '$\psi=2.5$','$\beta=0.965$','$\beta=0.97$'};

XX  = [
    1.001 31.500 0.975 0.85 0.30 3
    2.000 17.750 0.975 0.85 0.30 3
    2.500 11.350 0.975 0.85 0.30 3
    3.000  6.650 0.975 0.85 0.30 3
    2.500  7.300 0.975 0.90 0.30 3
    2.500 11.100 0.975 0.85 0.25 3
    2.500 11.330 0.975 0.85 0.3 2.5
    2.500 13.000 0.965 0.85 0.30 3
    2.500 12.200 0.970 0.85 0.30 3
    ];
XX  = [2.500 11.350 0.975 0.85 0.30 3];
nexp=size(XX,1);
strc    = [];
strd    = [];
strp    = [];
strcovc = [];
strcovd = [];
stracc  = [];
for iexp=1:nexp
    gam     = XX(iexp,1);
    alpha   = XX(iexp,2);
    beta    = XX(iexp,3);
    rhoh    = XX(iexp,4);
    rhol    = XX(iexp,5);
    psi     = XX(iexp,6);
    
    ncv0        = 3;                    % Order of approximation C: V former case
    ncv         = 3;                    % Order of approximation C: V
    if gam<1.01
        lfile       = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
    else
        lfile       = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
    end
    load(lfile)
    fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
    par1        = [gam rhoh rhol psi beta];
    par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
    [data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);

    load(lfile)
    paraml  = [alpha parl];
    paramh  = [alpha parh];
    param   = [alpha par];
    data.thvl= thvl;
    data.thvh= thvh;
    data.thpl= thpl;
    data.thph= thph;
    data.thv = thv;
    data.thp = thp;
    %% Moments+Accuracy Tables
    acc     = comp_acc(thv,thp,data,param);
    RRfp    = comp_rrfp(param,data,thv,thp);
    strd    = char(strd,sprintf('%s & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f && %5.2f & %5.2f & %5.2f \\\\',str{iexp},gam,alpha,RRfp.erf,RRfp.er,RRfp.eep,RRfp.srf,RRfp.sr,RRfp.sep));
    stracc  = char(stracc,sprintf('%s & %5.2f & %5.2f & %5.2f & %5.2f & %5.2f && %5.2f & %5.2f & %5.2f \\\\ ',str{iexp},gam,alpha,acc.ve1,acc.ve2,acc.vei,acc.pe1,acc.pe2,acc.pei));
end
%%
disp('Accuracy')
disp(stracc)
disp(' ')
disp('Return (Dividend Claim)')
disp(strd)
