% Generate Figure 2
clear all;
clc
%% Path + basic parameters
addpath ../method;
addpath ../data_last;
addpath decrules_last;
addpath sol_functions
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];

%% Parameters of approximation

nbx     = 10001;
xmin    =-0.2;
xmax    = 0.2;

%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Persistence
rhoh    = 0.85;
rhol    = 0.3;
psi     = 3;
beta    = 0.975;

gam     = 2.5;
alpha   = 11.35;
ncv0    = 3;						% Order of approximation C: V
ncv     = 3;						% Order of approximation C: V
par1        = [gam rhoh rhol psi beta];
par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
[data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);
TV4     = data.tv4d;
TV3     = data.tv3d;
%% Builds parameters of the model


paraml  = [alpha parl];
paramh  = [alpha parh];
param   = [alpha par];
lfile    = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
load(lfile)
data.thvl= thvl;
data.thvh= thvh;

accv     = resid_v(thv,param,data,1);

cd      = data.xd(:,1);
xxl     = data.xd(:,2);
xxh     = data.xd(:,3);
e       = data.xd(:,4);  %(Proba of rhol)
T       = length(cd);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
Pl      = param(17)*param(17);
Ph      = param(18)*param(18);

%% Distributions
x       = linspace(xmin,xmax,nbx)';
data.x  = x;
dx      = diff(x);
mx      = (x(2:nbx)+x(1:nbx-1))/2;


fgl     = zeros(nbx-1,nbx-1);
fgh     = zeros(nbx-1,nbx-1);
fdl     = zeros(nbx-1,nbx-1);
fdh     = zeros(nbx-1,nbx-1);
Etx     = zeros(T,1);
Vtx     = zeros(T,1);
Ebx     = zeros(T,1);
Vbx     = zeros(T,1);
Etx2    = zeros(T,1);
Vtx2    = zeros(T,1);
Ebx2    = zeros(T,1);
Vbx2    = zeros(T,1);

for t       = 1:T;
    data.xd     = [cd(t) xxl(t) xxh(t) e(t)];
    
    [twl,twh]   = ftwist(thv,param,data);
    
    fltmp       = exp(-0.5*(x-xxl(t)).*(x-xxl(t))/Pl)/sqrt(2*pi*Pl);
    flstmp      = fltmp.*twl;

    fltmp       = fltmp/sum(dx.*(fltmp(2:nbx)+fltmp(1:nbx-1))/2);
    fxl         = dx.*(fltmp(2:nbx)+fltmp(1:nbx-1))/2;

    flstmp      = flstmp/sum(dx.*(flstmp(2:nbx)+flstmp(1:nbx-1))/2);
    ftl         = dx.*(flstmp(2:nbx)+flstmp(1:nbx-1))/2;
    
    fhtmp       = exp(-0.5*(x-xxh(t)).*(x-xxh(t))/Ph)/sqrt(2*pi*Ph);
    fhstmp      = fhtmp.*twh;
    
    fhtmp       = fhtmp/sum(dx.*(fhtmp(2:nbx)+fhtmp(1:nbx-1))/2);
    fxh         = dx.*(fhtmp(2:nbx)+fhtmp(1:nbx-1))/2;
    
    fhstmp      = fhstmp/sum(dx.*(fhstmp(2:nbx)+fhstmp(1:nbx-1))/2);
    fth         = dx.*(fhstmp(2:nbx)+fhstmp(1:nbx-1))/2;
    
    Exl(t)      = mx'*fxl;
    Exh(t)      = mx'*fxh;
    Etxl(t)     = mx'*ftl;
    Etxh(t)     = mx'*fth;
    
    Fbx         = e(t)*fxl+(1-e(t))*fxh;
    Ftx         = e(t)*ftl+(1-e(t))*fth;
    Tw          = e(t)*twl+(1-e(t))*twh;
    
    Ebx(t)      = mx'*Fbx;     
    Vbx(t)      = (mx.*mx)'*Fbx-Ebx(t).*Ebx(t);
    
    Etx(t)      = mx'*Ftx;
    Vtx(t)      = (mx.*mx)'*Ftx-Etx(t).*Etx(t);
    
end
%%

time    = 1978:2011;
fs      = 12;
T       = length(cd);
HP  = hpmat(T,6.25);
hpc  = cd-HP\cd;
%% Figure 2(a)
subplot(211);[ax,h1,h2]=plotyy(time,[xxl xxh],time,hpc);
set(ax,'xlim',[1978 2011],'fontname','times','fontsize',fs,'ycolor','k')
set(h1(1),'color','k','linewidth',2,'linestyle','--')
set(h1(2),'color','k','linewidth',2)
% set(h1(3),'color','r','linew idth',2)
set(h2,'color',0.4*ones(1,3),'linewidth',2)
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig2a.eps
printpdf('figure2a');
close
%% Figure 2(b)
subplot(211);[ax,h1,h2]=plotyy(time,[Vbx Vtx],time,hpc);
set(ax,'xlim',[1978 2011],'fontname','times','fontsize',fs,'ycolor','k')
ylabel(ax(1),'Variance','fontname','times','fontsize',fs)
ylabel(ax(2),'$\widetilde{c}_{t}$','fontname','times','fontsize',fs,'interpreter','latex')
set(h1(1),'color','k','linewidth',2,'linestyle','--')
set(h1(2),'color','k','linewidth',2)
set(h2,'color',0.4*ones(1,3),'linewidth',2)
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig2b.eps
printpdf('figure2b');
close
