clear all;
clc
%% Path + basic parameters
addpath ../method;
addpath ../data_last;
addpath decrules_last;
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		                % sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];

%% Parameters of approximation

nbx     = 1001;
xmin    =-0.2;
xmax    = 0.2;
gmin    =-0.2;
gmax    = 0.2;
dmin    =-0.6;
dmax    = 0.6;

%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Persistence
rhoh    = 0.85;
rhol    = 0.3;
psi     = 3;
beta    = 0.975;
if rhoh==0.85
    gam     = 2.5;
    alpha   = 11.35;
    ncv0    = 3;						% Order of approximation C: V
    ncv     = 3;						% Order of approximation C: V
    par1        = [gam rhoh rhol psi beta];
    par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
    [data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);
else
    gam     = 3.25;
    alpha   = 3.63;
    ncv     = 4;						% Order of approximation C: V
    clear data dats par parl parh;
    [data,dats,par,parl,parh]   = build_params([gam rhoh rhol],[ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese]);
end
selyear = [1992 1999];
FB  = zeros(nbx-1,length(selyear));
FT  = zeros(nbx-1,length(selyear));
dat0=data;
for iexp=1:length(selyear)
    T=find([1978:2011]==selyear(iexp));
    %% Builds parameters of the model
    data=dat0;
    
    paraml  = [alpha parl];
    paramh  = [alpha parh];
    param   = [alpha par];
    lfile    = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
    load(lfile)
    data.thvl= thvl;
    data.thvh= thvh;
    data.thrl= thrl;
    data.thrh= thrh;
    data.thpl= thpl;
    data.thph= thph;
    data.thv = thv;
    data.thr = thr;
    data.thp = thp;
    
    cd      = data.xd(T,1);
    xxl     = data.xd(T,2);
    xxh     = data.xd(T,3);
    e       = data.xd(T,4);  %(Proba of rhol)
    rhol    = param(3);
    rhoh    = param(4);
    psil    = param(5);
    psih    = param(6);
    ggl     = param(7);
    ggh     = param(8);
    gdl     = param(9);
    gdh     = param(10);
    sxl     = param(11);
    sxh     = param(12);
    sgl     = param(13);
    sgh     = param(14);
    sdl     = param(15);
    sdh     = param(16);
    Pl      = param(17)*param(17);
    Ph      = param(18)*param(18);
    
    %% Distributions
    x       = linspace(xmin,xmax,nbx)';
    data.x  = x;
    dx      = diff(x);
    mx      = (x(2:nbx)+x(1:nbx-1))/2;
    g       = linspace(gmin,gmax,nbx)';
    dg      = diff(g);
    d       = linspace(dmin,dmax,nbx)';
    dd      = diff(d);
    mg      = (g(2:nbx)+g(1:nbx-1))/2;
    md      = (d(2:nbx)+d(1:nbx-1))/2;
    fgl     = zeros(nbx-1,nbx-1);
    fgh     = zeros(nbx-1,nbx-1);
    fdl     = zeros(nbx-1,nbx-1);
    fdh     = zeros(nbx-1,nbx-1);
    t       = 1;
    data.xd     = [cd xxl xxh e];
    [twl,twh]   = ftwist(thv,param,data);
    
    fltmp       = exp(-0.5*(x-xxl(t)).*(x-xxl(t))/Pl)/sqrt(2*pi*Pl);
    fltmp       = fltmp/sum(dx.*(fltmp(2:nbx)+fltmp(1:nbx-1))/2);
    flstmp      = fltmp.*twl;
    flstmp      = flstmp/sum(dx.*(flstmp(2:nbx)+flstmp(1:nbx-1))/2);
    fxl         = dx.*(fltmp(2:nbx)+fltmp(1:nbx-1))/2;
    ftl         = dx.*(flstmp(2:nbx)+flstmp(1:nbx-1))/2;
    
    fhtmp       = exp(-0.5*(x-xxh(t)).*(x-xxh(t))/Ph)/sqrt(2*pi*Ph);
    fhtmp       = fhtmp/sum(dx.*(fhtmp(2:nbx)+fhtmp(1:nbx-1))/2);
    fxh         = dx.*(fhtmp(2:nbx)+fhtmp(1:nbx-1))/2;
    fhstmp      = fhtmp.*twh;
    fhstmp      = fhstmp/sum(dx.*(fhstmp(2:nbx)+fhstmp(1:nbx-1))/2);
    fth         = dx.*(fhstmp(2:nbx)+fhstmp(1:nbx-1))/2;
    
    fg          = zeros(nbx-1,nbx-1);
    for i=1:nbx-1
        gltmp   = exp(-0.5*(g-ggl-rhol*mx(i)).*(g-ggl-rhol*mx(i))/(sxl*sxl+sgl*sgl))/sqrt(2*pi*(sxl*sxl+sgl*sgl));
        gltmp   = gltmp/sum(dg.*(gltmp(2:nbx)+gltmp(1:nbx-1))/2);
        fgl(:,i)= dg.*(gltmp(2:nbx)+gltmp(1:nbx-1))/2;
        
        ghtmp   = exp(-0.5*(g-ggh-rhoh*mx(i)).*(g-ggh-rhoh*mx(i))/(sxh*sxh+sgh*sgh))/sqrt(2*pi*(sxh*sxh+sgh*sgh));
        ghtmp   = ghtmp/sum(dg.*(ghtmp(2:nbx)+ghtmp(1:nbx-1))/2);
        fgh(:,i)= dg.*(ghtmp(2:nbx)+ghtmp(1:nbx-1))/2;
        
        dltmp   = exp(-0.5*(d-gdl-rhol*psil*mx(i)).*(d-gdl-rhol*psil*mx(i))/(psil*psil*sxl*sxl+sdl*sdl))/sqrt(2*pi*(psil*psil*sxl*sxl+sdl*sdl));
        dltmp   = dltmp/sum(dd.*(dltmp(2:nbx)+dltmp(1:nbx-1))/2);
        fdl(:,i)= dd.*(dltmp(2:nbx)+dltmp(1:nbx-1))/2;
        
        dhtmp   = exp(-0.5*(d-gdh-rhoh*psih*mx(i)).*(d-gdh-rhoh*psih*mx(i))/(psih*psih*sxh*sxh+sdh*sdh))/sqrt(2*pi*(psih*psih*sxh*sxh+sdh*sdh));
        dhtmp   = dhtmp/sum(dd.*(dhtmp(2:nbx)+dhtmp(1:nbx-1))/2);
        fdh(:,i)= dd.*(dhtmp(2:nbx)+dhtmp(1:nbx-1))/2;
    end;
    
    Fbx = e*fxl+(1-e)*fxh;
    Ftx = e*ftl+(1-e)*fth;
    
    Fbgl = fgl*fxl;
    Fbgh = fgh*fxh;
    Ftgl = fgl*ftl;
    Ftgh = fgh*fth;
    
    Fbg  = e*Fbgl+(1-e)*Fbgh;
    Ftg  = e*Ftgl+(1-e)*Ftgh;
    
    Fbdl = fdl*fxl;
    Fbdh = fdh*fxh;
    Ftdl = fdl*ftl;
    Ftdh = fdh*fth;
    
    Fbd  = e*Fbdl+(1-e)*Fbdh;
    Ftd  = e*Ftdl+(1-e)*Ftdh;
    
    
    Ebx  = mx'*Fbx;
    Sbx  = sqrt((mx.*mx)'*Fbx-Ebx*Ebx);
    E2bx = ((mx-Ebx).*(mx-Ebx))'*Fbx;
    E3bx = ((mx-Ebx).*(mx-Ebx).*(mx-Ebx))'*Fbx;
    E4bx = ((mx-Ebx).*(mx-Ebx).*(mx-Ebx).*(mx-Ebx))'*Fbx;
    Skbx = E3bx/(Sbx^3);
    Kubx = E4bx/(E2bx*E2bx)-3;
    
    Etx  = mx'*Ftx;
    Stx  = sqrt((mx.*mx)'*Ftx-Etx*Etx);
    E2tx = ((mx-Etx).*(mx-Etx))'*Ftx;
    E3tx = ((mx-Etx).*(mx-Etx).*(mx-Etx))'*Ftx;
    E4tx = ((mx-Etx).*(mx-Etx).*(mx-Etx).*(mx-Etx))'*Ftx;
    Sktx = E3tx/(Stx^3);
    Kutx = E4tx/(E2tx*E2tx)-3;
    
    
    
    Ebg  = mg'*Fbg;
    Sbg  = sqrt((mg.*mg)'*Fbg-Ebg*Ebg);
    E2bg = ((mg-Ebg).*(mg-Ebg))'*Fbg;
    E3bg = ((mg-Ebg).*(mg-Ebg).*(mg-Ebg))'*Fbg;
    E4bg = ((mg-Ebg).*(mg-Ebg).*(mg-Ebg).*(mg-Ebg))'*Fbg;
    Skbg = E3bg/(Sbg^3);
    Kubg = E4bg/(E2bg*E2bg)-3;
    
    Etg  = mg'*Ftg;
    Stg  = sqrt((mg.*mg)'*Ftg-Etg*Etg);
    E2tg = ((mg-Etg).*(mg-Etg))'*Ftg;
    E3tg = ((mg-Etg).*(mg-Etg).*(mg-Etg))'*Ftg;
    E4tg = ((mg-Etg).*(mg-Etg).*(mg-Etg).*(mg-Etg))'*Ftg;
    Sktg = E3tg/(Stg^3);
    Kutg = E4tg/(E2tg*E2tg)-3;
    
    Ebd  = md'*Fbd;
    Sbd  = sqrt((md.*md)'*Fbd-Ebd*Ebd);
    E2bd = ((md-Ebd).*(md-Ebd))'*Fbd;
    E3bd = ((md-Ebd).*(md-Ebd).*(md-Ebd))'*Fbd;
    E4bd = ((md-Ebd).*(md-Ebd).*(md-Ebd).*(md-Ebd))'*Fbd;
    Skbd = E3bd/(Sbd^3);
    Kubd = E4bd/(E2bd*E2bd)-3;
    
    Etd  = md'*Ftd;
    Std  = sqrt((md.*md)'*Ftd-Etd*Etd);
    E2td = ((md-Etd).*(md-Etd))'*Ftd;
    E3td = ((md-Etd).*(md-Etd).*(md-Etd))'*Ftd;
    E4td = ((md-Etd).*(md-Etd).*(md-Etd).*(md-Etd))'*Ftd;
    Sktd = E3td/(Std^3);
    Kutd = E4td/(E2td*E2td)-3;
    
    fprintf('%7.4f & %7.4f \n',Ebx,Sbx)
    fprintf('%7.4f & %7.4f \n',Etx,Stx)
    fprintf('%7.4f & %7.4f \n',Skbx,Kubx)
    fprintf('%7.4f & %7.4f \n',Sktx,Kutx)
        
    
    fprintf('%7.4f & %7.4f \n',Ebg,Sbg)
    fprintf('%7.4f & %7.4f \n',Etg,Stg)
    fprintf('%7.4f & %7.4f \n',Skbg,Kubg)
    fprintf('%7.4f & %7.4f \n',Sktg,Kutg)
    
    
    fprintf('%7.4f & %7.4f \n',Ebd,Sbd)
    fprintf('%7.4f & %7.4f \n',Etd,Std)
    fprintf('%7.4f & %7.4f \n',Skbd,Kubd)
    fprintf('%7.4f & %7.4f \n',Sktd,Kutd)
    
    FB(:,iexp)=Fbx;
    FT(:,iexp)=Ftx;
end
%% Figure 3
for i=1:2
    subplot(2,2,i)
    h=plot(mx,[FB(:,i) FT(:,i)]);
    set(h(1),'color','k','linewidth',2,'linestyle','--');
    set(h(2),'color','k','linewidth',2);
    xlabel('x_t','fontname','times','fontsize',16)
    title(selyear(i),'fontname','times','fontsize',12)
%     if i==1;
%     o=legend('Bayesian','Twisted');
%     set(gca,'fontname','times','fontsize',10)
%     end
    set(gca,'fontname','times','fontsize',12,'ylim',[0 0.007],'ytick',[])
end
pause
print -depsc2 fig3.eps
printpdf('figure3');
close