% Compute the decomposition 
%
% E(R)=E[(P'+D')/D]=E[(1+z')*d']=cov[(1+z'),d]-E[1+z']E(d']
%
% and prepare Figure 6b cov(z',d')
clear all;
%% Path + basic parameters
addpath ../method
addpath ../data_last
addpath decrules_last
save_   = 1;                        % 1 -> save results
crit    = 1e-6;		  
% sum of abs. values small enough to be a solution
delta   = 1e-8;		                % differencing interval for numerical gradient
alpha   = 1e-3;		                % tolerance on rate of descent
itmax   = 1000;		                % max no. of iterations
dispo   = 1;                        % partial display
options = [itmax;crit;delta;alpha;dispo];
%% Parameters of approximation
nxlv    = 2;						% Order of approximation X_hat L: V
nxhv    = 2;						% Order of approximation X_hat H: V
nev     = 2;						% Order of approximation E: V

nca     = 5;						% Order of approximation C: V
nxla    = 2;						% Order of approximation X_hat L: V
nxha    = 2;						% Order of approximation X_hat H: V
nea     = 2;						% Order of approximation E: V

ncr     = 3;						% Order of approximation C: R
nxlr    = 3;						% Order of approximation X_hat L: R
nxhr    = 3;						% Order of approximation X_hat H: R
ner     = 3;						% Order of approximation E: R

ncp     = 3;						% Order of approximation C: P
nxlp    = 3;						% Order of approximation X_hat L: P
nxhp    = 3;						% Order of approximation X_hat H: P
nep     = 3;						% Order of approximation E: P

nodesc	= 8;    					% Number of nodes for C
nodesxl	= 8;						% Number of nodes for X_hat L
nodesxh	= 8;    					% Number of nodes for X_hat H
nodese	= 8;						% Number of nodes for E

%% Structural parameters of the model
Epd     = 45.5;26.56;
spd     = 0.29;
k0      = log(1+Epd)-Epd*log(Epd)/(1+Epd);
k1      = Epd/(1+Epd);
XX  = [
    2.500 11.350 0.975 0.85 0.30 3
    ];
nexp=size(XX,1);
strc    = [];
strd    = [];
strp    = [];
strcovc = [];
strcovd = [];
stracc  = [];
for iexp=1:nexp
    gam     = XX(iexp,1);
    alpha   = XX(iexp,2);
    beta    = XX(iexp,3);
    rhoh    = XX(iexp,4);
    rhol    = XX(iexp,5);
    psi     = XX(iexp,6);
    
    ncv0        = 3;                    % Order of approximation C: V former case
    ncv         = 3;                    % Order of approximation C: V
    if gam<1.01
        lfile       = sprintf('decrules_last/res_log_%d_%d_%d_%d',100*psi,100*rhoh,100*rhol,1000*beta);
    else
        lfile       = sprintf('decrules_last/res_%d_%d_%d_%d_%d',100*gam,100*psi,100*rhoh,100*rhol,1000*beta);
    end
    load(lfile)
    fprintf('gamma = %4.2f , alpha = %4.2f\n',gam,alpha);
    par1        = [gam rhoh rhol psi beta];
    par2        = [ncv nxlv nxhv nev ncr nxlr nxhr ner ncp nxlp nxhp nep nodesc nodesxl nodesxh nodese nca nxla nxha nea];
    [data,dats,par,parl,parh]   = build_params(par1,par2,ncv0);
    load(lfile)
    paraml  = [alpha parl];
    paramh  = [alpha parh];
    param   = [alpha par];
    data.thvl= thvl;
    data.thvh= thvh;
    data.thpl= thpl;
    data.thph= thph;
    data.thv = thv;
    data.thp = thp;
    declog  = comp_decomp_log(param,data,thp);
end
%%
time	= 1978:2011;
clzld   = declog.er-declog.ed.*declog.ez;
subplot(2,1,1);
h1  = plot(time,clzld);set(h1,'color','k','linewidth',2)
set(gca,'fontname','times','fontsize',12,'xlim',[time(1) time(end)],'ylim',mean(clzld)*[1.5 0.5]);
dates=[1980 1981 1982 1990 1991 2001 2007 2008 2009]; % Years with recessions
ylim=get(gca,'ylim');
for i=1:length(dates)
    h=line(dates(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
pause
printpdf('figure6b');
print -depsc2 fig6b.eps
close