clear all
addpath data
addpath ../method
T0      = 1978;
T1      = 2011;
data_basic;
lc=log(DATA(find(DATA==T0):end,5));
T=length(lc);
hpc=lc-hpfilter(lc,6.25);
xhat    = csvread('data_hat_tilde.csv',1,1);

data_indices;
full    = DATA(:,1);
smpl    = find(full==T0):find(full==T1);
Rdata   = DATA(smpl,2);
Rfdata  = DATA(smpl,3);
PDdata  = DATA(smpl,4);
BLdata  = DATA(smpl,5);
JLdata  = DATA(smpl,6);
MUdata  = DATA(smpl,7);

data_model;
full    = Ym(:,1);
smpl    = find(full==T0):find(full==T1);
Rmodel  = Ym(:,2);
Rfmodel = Ym(:,3);
EPmodel = Ym(:,4);
PDmodel = Ym(:,5);
VRmodel = Ym(:,6);

time    = 1978:2011;
fs      = 12;
gray    = 0.5*ones(1,3);
dat_rec = [1980 1981 1982 1990 1991 2001 2007 2008 2009]; % Years with recessions
nd      = length(dat_rec);

%% Figure 7
% Figure 7(a)
subplot(211);h=plot(time,[Rmodel-Rfmodel Rdata-Rfdata]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-40 40];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1978 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig7a.eps
printpdf('figure7a');
close

% Figure 7(b)
subplot(211);h=plot(time,[Rfmodel Rfdata]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-10 10];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1978 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig7b.eps
printpdf('figure7b');
close

% Figure 7(c)
subplot(211);h=plot(time,[PDmodel PDdata]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[0 100];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1978 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig7c.eps
printpdf('figure7c');
close

% Figure 7(d)
y1=VRmodel/mean(VRmodel); y1=(y1-mean(y1))/std(y1);
y2=BLdata/mean(BLdata);   y2=(y2-mean(y2))/std(y2);
subplot(211);h=plot(time,[y1 y2]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-4 4];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1978 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig7d.eps
printpdf('figure7d');
close

%% Figure 8
% Figure 8(a)
time=1979:2011;
y1=EPmodel(2:end)/mean(EPmodel(2:end));   y1=(y1-mean(y1))/std(y1);
y2=JLdata(1:end-1)/mean(JLdata(1:end-1)); y2=(y2-mean(y2))/std(y2);
subplot(211);h=plot(time,[y1 y2]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-4 4];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1979 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig8a.eps
printpdf('figure8a');
close

% Figure 8(b)
time=1980:2011;
y1=diff(EPmodel(2:end)); y1=y1/mean(y1); y1=(y1-mean(y1))/std(y1);
y2=diff(JLdata(1:end-1));y2=y2/mean(y2); y2=(y2-mean(y2))/std(y2);
subplot(211);h=plot(time,[y1 y2]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-4 4];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1979 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig8b.eps
printpdf('figure8b');
close

% Figure 8(c)
time=1979:2011;
y1=EPmodel(2:end)/mean(EPmodel(2:end))-1; 
y2=MUdata(1:end-1)/mean(MUdata(1:end-1))-1;
subplot(211);h=plot(time,[y1 y2]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-1 1];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1979 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig8c.eps
printpdf('figure8c');
close

% Figure 8(d)
time=1980:2011;
y1=EPmodel(2:end); y1=diff(y1/mean(y1)-1);
y2=MUdata(1:end-1);y2=diff(y2/mean(y2)-1);
subplot(211);h=plot(time,[y1 y2]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color',gray,'linewidth',2)
ylim =[-1 1];
for i=1:nd
    h=line(dat_rec(i)*ones(1,2),ylim);
    set(h,'linestyle','--','color','k')
end
set(gca,'xlim',[1979 2011],'ylim',ylim,'fontname','times','fontsize',fs,'ycolor','k')
xlabel('Years','fontname','times','fontsize',fs)
pause
print -depsc2 fig8d.eps
printpdf('figure8d');
close

%% Figure 9
k   = 8;
cd  = zeros(2*k+1,1);
cm  = zeros(2*k+1,1);
tmp = corrcoef([JLdata log(PDdata)  log(PDmodel)]);
cd(k+1)=tmp(1,2);
cm(k+1)=tmp(1,3);
for i=1:8
    tmp = corrcoef([JLdata(i+1:end) log(PDdata(1:end-i))  log(PDmodel(1:end-i))]);
    cd(k+1-i)=tmp(1,2);
    cm(k+1-i)=tmp(1,3);
    tmp = corrcoef([JLdata(1:end-i) log(PDdata(i+1:end))  log(PDmodel(i+1:end))]);
    cd(k+1+i)=tmp(1,2);
    cm(k+1+i)=tmp(1,3);
end
subplot(211);h=plot(-8:8,[cm cd]);
set(h(1),'color','k','linewidth',2)
set(h(2),'color','k','linewidth',2,'linestyle','--')
set(gca,'xlim',[-8 8],'ylim',[-1 1],'fontname','times','fontsize',fs,'ycolor','k')
pause
print -depsc2 fig9.eps
printpdf('figure9');
close
