function res = comp_decomp(param,data,thp)
% Compute the decomposition proposed by the first referee at AER
alpha   = param(1);
beta    = param(2);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
spl     = param(17);
sph     = param(18);
coefxl  = param(19);
coefxh  = param(20);
coefc   = param(21);
coefe   = param(22);
accprob = param(23);
gam     = param(24);

spi     = sqrt(pi);
spi3    = spi*spi*spi;


thpl    = data.thpl;
thph    = data.thph;
XX      = data.xd;
TP4     = data.tp4d;
y       = data.y1d;
w       = data.w1d;
int3    = data.y3d;
wint3   = data.w3d;
kh      = data.kh;
kl      = data.kl;
fh      = data.fh;
fl      = data.fl;
nint3   = size(int3,1);
nint    = size(w,1);

c       = XX(:,1);
xl      = XX(:,2);
xh      = XX(:,3);
e       = XX(:,4);
Tc      = size(c,1);
HP      = hpmat(Tc,6.25);
hpc     = c-HP\c;


prh     = 0;
prl     = 0;
prh2    = 0;
prl2    = 0;
pRh     = 0;
pRl     = 0;
pRh2    = 0;
pRl2    = 0;
pzh     = 0;
pzl     = 0;
pzh2    = 0;
pzl2    = 0;
pdh     = 0;
pdl     = 0;
pdh2    = 0;
pdl2    = 0;
for i=1:nint;    
    yi      = y(i);
    wi      = w(i);
    tmpRh   = 0;
    tmpRl   = 0;
    tmpRh2  = 0;
    tmpRl2  = 0;
    tmprh   = 0;
    tmprl   = 0;
    tmprh2  = 0;
    tmprl2  = 0;
    tmpzh   = 0;
    tmpzl   = 0;
    tmpzh2  = 0;
    tmpzl2  = 0;
    tmpdh   = 0;
    tmpdl   = 0;
    tmpdh2  = 0;
    tmpdl2  = 0;

    for j=1:nint3;
        gpl     = ggl+rhol*(xl+spl*yi)+sgl*int3(j,2)+sxl*int3(j,1);
        gph     = ggh+rhoh*(xh+sph*yi)+sgh*int3(j,2)+sxh*int3(j,1);
        gdpl    = gdl+psil*rhol*(xl+spl*yi)+sdl*int3(j,3)+psil*sxl*int3(j,1);
        gdph    = gdh+psih*rhoh*(xh+sph*yi)+sdh*int3(j,3)+psih*sxh*int3(j,1);
        %
        % nupij: i-> subscript; j->superscript
        %
        nupll   = [gpl-ggl-rhol*xl gdpl-gdl-psil*rhol*xl];
        nuphh   = [gph-ggh-rhoh*xh gdph-gdh-psih*rhoh*xh];
        nuphl   = [gpl-ggh-rhoh*xh gdpl-gdh-psih*rhoh*xh];
        nuplh   = [gph-ggl-rhol*xl gdph-gdl-psil*rhol*xl];
        %
        % compute the etas
        %
        l1      = likelihood(nupll,fl);
        l2      = likelihood(nuphl,fh);
        etapl   = e.*l1./(e.*l1+(1-e).*l2);
        
        l1      = likelihood(nuplh,fl);
        l2      = likelihood(nuphh,fh);
        etaph   = e.*l1./(e.*l1+(1-e).*l2);
        
        zxll    = coefxl*(rhol*xl+nupll*kl');
        zxlh    = coefxl*(rhol*xl+nuplh*kl');
        
        zxhh    = coefxh*(rhoh*xh+nuphh*kh');
        zxhl    = coefxh*(rhoh*xh+nuphl*kh');
                
        zel     = coefe*norm_inv(etapl);
        zeh     = coefe*norm_inv(etaph);

        zch      = coefc*(c+gph);
        zcl      = coefc*(c+gpl);
        
        Tpl     = makepoly4_p([zcl,zxll,zxhl,zel]);
        Tpl2    = makepoly3_p([zcl,zxll,zxhl]);

        Tph     = makepoly4_p([zch,zxlh,zxhh,zeh]);
        Tph2    = makepoly3_p([zch,zxlh,zxhh]);
              
        prpl                     = (Tpl*thp);
        prpl(etapl<accprob)      = (Tpl2(etapl<accprob,:)*thph);
        prpl(etapl>(1-accprob))  = (Tpl2(etapl>(1-accprob),:)*thpl);
        
        prph                     = (Tph*thp);
        prph(etaph<accprob)      = (Tph2(etaph<accprob,:)*thph);
        prph(etaph>(1-accprob))  = (Tph2(etaph>(1-accprob),:)*thpl);
        lrh     = prph+gdph;
        lrl     = prpl+gdpl;
        tmpRh   = tmpRh+lrh*wint3(j)/spi3;
        tmpRl   = tmpRl+lrl*wint3(j)/spi3;
        tmpRh2  = tmpRh2+(lrh.*lrh)*wint3(j)/spi3;
        tmpRl2  = tmpRl2+(lrl.*lrl)*wint3(j)/spi3;
        tmprh   = tmprh+(prph.*(gdph))*wint3(j)/spi3;
        tmprl   = tmprl+(prpl.*(gdpl))*wint3(j)/spi3;
        tmprh2  = tmprh2+(prph.*(gdph)).^2*wint3(j)/spi3;
        tmprl2  = tmprl2+(prpl.*(gdpl)).^2*wint3(j)/spi3;
        tmpzh   = tmpzh+prph*wint3(j)/spi3;
        tmpzl   = tmpzl+prpl*wint3(j)/spi3;
        tmpzh2  = tmpzh2+prph.^2*wint3(j)/spi3;
        tmpzl2  = tmpzl2+prpl.^2*wint3(j)/spi3;
        tmpdh   = tmpdh+(gdph)*wint3(j)/spi3;
        tmpdl   = tmpdl+(gdpl)*wint3(j)/spi3;
        tmpdh2  = tmpdh2+(gdph.^2)*wint3(j)/spi3;
        tmpdl2  = tmpdl2+(gdpl.^2)*wint3(j)/spi3;

    end
    
    prh     = prh+tmprh*wi/spi;
    prl     = prl+tmprl*wi/spi;
    prh2    = prh2+tmprh2*wi/spi;
    prl2    = prl2+tmprl2*wi/spi;
    pRh     = pRh+tmpRh*wi/spi;
    pRl     = pRl+tmpRl*wi/spi;
    pRh2    = pRh2+tmpRh2*wi/spi;
    pRl2    = pRl2+tmpRl2*wi/spi;
    pzh     = pzh+tmpzh*wi/spi;
    pzl     = pzl+tmpzl*wi/spi;
    pzh2    = pzh2+tmpzh2*wi/spi;
    pzl2    = pzl2+tmpzl2*wi/spi;
    pdh     = pdh+tmpdh*wi/spi;
    pdl     = pdl+tmpdl*wi/spi; 
    pdh2    = pdh2+tmpdh2*wi/spi;
    pdl2    = pdl2+tmpdl2*wi/spi; 
    
end
res.r   = exp(TP4*thp);
res.eR2 = e.*pRl2+(1-e).*pRh2;
res.eR  = e.*pRl+(1-e).*pRh;
res.er2 = e.*prl2+(1-e).*prh2;
res.er  = e.*prl+(1-e).*prh;
res.ez  = e.*pzl+(1-e).*pzh;
res.ez2 = e.*pzl2+(1-e).*pzh2;
res.ed  = e.*pdl+(1-e).*pdh;
res.ed2 = e.*pdl2+(1-e).*pdh2;
res.cov = res.er-res.ez.*res.ed;

res.Eer = mean(res.er);
res.Eez = mean(res.ez);
res.Eed = mean(res.ed);
res.Ecov= mean(res.cov);
