function [twistl,twisth]=ftwist(thv,param,data)

alpha   = param(1);
rhol    = param(3);
rhoh    = param(4);
psil    = param(5);
psih    = param(6);
ggl     = param(7);
ggh     = param(8);
gdl     = param(9);
gdh     = param(10);
sxl     = param(11);
sxh     = param(12);
sgl     = param(13);
sgh     = param(14);
sdl     = param(15);
sdh     = param(16);
spl     = param(17);
sph     = param(18);
coefxl  = param(19);
coefxh  = param(20);
coefc   = param(21);
coefe   = param(22);
accprob = param(23);

spi     = sqrt(pi);
spi3    = spi*spi*spi;

thvl    = data.thvl;
thvh    = data.thvh;
x       = data.x;
XX      = data.xd;
y       = data.y1d;
w       = data.w1d;
INT3    = data.y3d;
wint3   = data.w3d;
Kh      = data.kh;
Kl      = data.kl;
Fh      = data.fh;
Fl      = data.fl;
nint    = size(w,1);
nint3   = size(INT3,1);


C   = XX(:,1);
Xl  = XX(:,2);
Xh  = XX(:,3);
E   = XX(:,4);
tmppsil = 0;
tmppsih = 0;
for i=1:nint;
    
    yi      = y(i);
    wi      = w(i);
    innerl  = 0;
    innerh  = 0;
    for j=1:nint3;
        gpl     = ggl+rhol*(Xl+spl*yi)+sgl*INT3(j,2)+sxl*INT3(j,1);
        gph     = ggh+rhoh*(Xh+sph*yi)+sgh*INT3(j,2)+sxh*INT3(j,1);
        gdpl    = gdl+psil*rhol*(Xl+spl*yi)+sdl*INT3(j,3)+psil*sxl*INT3(j,1);
        gdph    = gdh+psih*rhoh*(Xh+sph*yi)+sdh*INT3(j,3)+psih*sxh*INT3(j,1);
        %
        % nupij: i-> subscript; j->superscript
        %
        nupll   = [gpl-ggl-rhol*Xl gdpl-gdl-psil*rhol*Xl];
        nuphh   = [gph-ggh-rhoh*Xh gdph-gdh-psih*rhoh*Xh];
        nuphl   = [gpl-ggh-rhoh*Xh gdpl-gdh-psih*rhoh*Xh];
        nuplh   = [gph-ggl-rhol*Xl gdph-gdl-psil*rhol*Xl];
        %
        % Compute the etas
        %
        L1      = likelihood(nupll,Fl);
        L2      = likelihood(nuphl,Fh);
        etapl   = E.*L1./(E.*L1+(1-E).*L2);
        
        L1      = likelihood(nuplh,Fl);
        L2      = likelihood(nuphh,Fh);
        etaph   = E.*L1./(E.*L1+(1-E).*L2);
        
        zxll    = (rhol*Xl+nupll*Kl')*coefxl;
        zxlh    = (rhol*Xl+nuplh*Kl')*coefxl;
        
        zxhl    = (rhoh*Xh+nuphl*Kh')*coefxh;
        zxhh    = (rhoh*Xh+nuphh*Kh')*coefxh;
        
        zel     = norm_inv(etapl)*coefe;
        zeh     = norm_inv(etaph)*coefe;
        
        zcl     = (C+gpl)*coefc;
        zch     = (C+gph)*coefc;
        
        Tvl     = makepoly4_v([zcl,zxll,zxhl,zel]);
        Tvl2    = makepoly3_v([zcl,zxll,zxhl]);

        Tvh     = makepoly4_v([zch,zxlh,zxhh,zeh]);
        Tvh2    = makepoly3_v([zch,zxlh,zxhh]);
    
        vtpl                    = exp(Tvl*thv);
        vtpl(etapl<accprob)     = exp(Tvl2(etapl<accprob,:)*thvh);
        vtpl(etapl>(1-accprob)) = exp(Tvl2(etapl>(1-accprob),:)*thvl);
        
        vtph                    = exp(Tvh*thv);
        vtph(etaph<accprob)     = exp(Tvh2(etaph<accprob,:)*thvh);
        vtph(etaph>(1-accprob)) = exp(Tvh2(etaph>(1-accprob),:)*thvl);
        
        innerl  = innerl+vtpl*wint3(j)/spi3;
        innerh  = innerh+vtph*wint3(j)/spi3;
        
    end
    tmpxl   = exp(-alpha*innerl);
    tmpxh   = exp(-alpha*innerh);
    
    tmppsil = tmppsil+tmpxl*wi/spi;
    tmppsih = tmppsih+tmpxh*wi/spi;
end
PSI     = E.*tmppsil+(1-E).*tmppsih;

innerl  =0;
innerh  =0;

for j=1:nint3;
    gpl     = ggl+rhol*x(:)+sgl*INT3(j,2)+sxl*INT3(j,1);
    gph     = ggh+rhoh*x(:)+sgh*INT3(j,2)+sxh*INT3(j,1);
    gdpl    = gdl+psil*rhol*x(:)+sdl*INT3(j,3)+psil*sxl*INT3(j,1);
    gdph    = gdh+psih*rhoh*x(:)+sdh*INT3(j,3)+psih*sxh*INT3(j,1);
    %
    % nupij: i-> subscript; j->superscript
    %        
    nupll   = [gpl-ggl-rhol*Xl gdpl-gdl-psil*rhol*Xl];
    nuphh   = [gph-ggh-rhoh*Xh gdph-gdh-psih*rhoh*Xh];
    nuphl   = [gpl-ggh-rhoh*Xh gdpl-gdh-psih*rhoh*Xh];
    nuplh   = [gph-ggl-rhol*Xl gdph-gdl-psil*rhol*Xl];
    %
    % Compute the etas
    %
    L1      = likelihood(nupll,Fl);
    L2      = likelihood(nuphl,Fh);
    etapl   = E.*L1./(E.*L1+(1-E).*L2);
    
    L1      = likelihood(nuplh,Fl);
    L2      = likelihood(nuphh,Fh);
    etaph   = E.*L1./(E.*L1+(1-E).*L2);
    
    zxll    = (rhol*Xl+nupll*Kl')*coefxl;
    zxlh    = (rhol*Xl+nuplh*Kl')*coefxl;
    
    zxhl    = (rhoh*Xh+nuphl*Kh')*coefxh;
    zxhh    = (rhoh*Xh+nuphh*Kh')*coefxh;
    
    zel     = norm_inv(etapl)*coefe;
    zeh     = norm_inv(etaph)*coefe;
    
    zcl     = (C+gpl)*coefc;
    zch     = (C+gph)*coefc;
    
    Tvl     = makepoly4_v([zcl,zxll,zxhl,zel]);
    Tvl2    = makepoly3_v([zcl,zxll,zxhl]);
    
    Tvh     = makepoly4_v([zch,zxlh,zxhh,zeh]);
    Tvh2    = makepoly3_v([zch,zxlh,zxhh]);
    
    vtpl                    = exp(Tvl*thv);
    vtpl(etapl<accprob)     = exp(Tvl2(etapl<accprob,:)*thvh);
    vtpl(etapl>(1-accprob)) = exp(Tvl2(etapl>(1-accprob),:)*thvl);
    
    vtph                    = exp(Tvh*thv);
    vtph(etaph<accprob)     = exp(Tvh2(etaph<accprob,:)*thvh);
    vtph(etaph>(1-accprob)) = exp(Tvh2(etaph>(1-accprob),:)*thvl);
    
    innerl  = innerl+vtpl*wint3(j)/spi3;
    innerh  = innerh+vtph*wint3(j)/spi3;
end


twistl  = exp(-alpha*innerl)./PSI;
twisth  = exp(-alpha*innerh)./PSI;
