function [a,v,LLF,l,P]=kf_bivariate_bansalyaron(y,mu,Z,T,R,H,Q,priorMean,priorCovariance)
%
% Usage 
% [a,LLF,l]=kf_bivariate_bansalyaron(y,Z,T,R,H,Q,priorMean,priorCovariance)
%
% y         - 2 by T data
% Z         - 2 by 1 vector of loadings on x
% T         - 1 by 1 vector of AR
% R         - 1 by 1 scalar, shoudl usually by 1
% H         - 2 by 2 covariance of observable shocks
% Q         - 1 by 1 variance of latent process shock
% priorMean - 1 by 1 prior mean for a(0)
% priorCovariance - 1 by 1 prior covariance for a(0)
%
%
% Output
% a         - 2 by T+1 conditional latent process means
% LLF       - bivariate normal log-likelihood
% l         - T by 1 vector of log-likelihoods


[p,tau]=size(y);
m=1;
a=zeros(m,tau+1);
v=zeros(p,tau);

a(1,:)=priorMean;
P=priorCovariance;

l=zeros(tau,1);
llfcont=-log(2*pi);
for t=1:tau
    v(:,t)=y(:,t)-mu-Z*a(:,t);
    F=Z*P*Z'+H;
    K=T*P*Z'*inv(F);
    L=T-K*Z;
    a(:,t+1)=T*a(:,t)+K*v(:,t);
    P=T*P*L'+R*Q*R';
    l(t)=llfcont-0.5*(log(det(F))+v(:,t)'*inv(F)*v(:,t));
end
LLF=sum(l);
