function [LLF,l,a,P]=kf_bivariate_bansalyaron_wraper_inference(variable_parameters,fixed_parameters,is_variable,y,priorMean,priorCovariance)
%
% Usage
% [a,LLF,l]=kf_bivariate_bansalyaron(y,Z,T,R,H,Q,priorMean,priorCovariance)
%
% y         - 2 by T data
% Order is mu(1) mu(2) Z(1) Z(2) fisher(T(1)) log(H(1,1)) log(H(2,2)) log(Q)
% Z         - 2 by 1 vector of loadings on x
% T         - 1 by 1 vector of AR
% R         - 1 by 1 scalar, shoudl usually by 1
% H         - 2 by 2 covariance of observable shocks
% Q         - 1 by 1 variance of latent process shock
% priorMean - 1 by 1 prior mean for a(0)
% priorCovariance - 1 by 1 prior covariance for a(0)
%
%
% Output
% a         - 2 by T+1 conditional latent process means
% LLF       - bivariate normal log-likelihood
% l         - T by 1 vector of log-likelihoods


variable_count=1;
fixed_count=1;

mu=zeros(2,1);
Z=zeros(2,1);
T=zeros(1);
R=1;
H=zeros(2,2);
Q=zeros(1);

var_list={'mu(1)','mu(2)','Z(1)','Z(2)','T','H(1,1)','H(2,2)','Q'};
for i=1:length(is_variable)
    if is_variable(i)
        eval([var_list{i} '=variable_parameters(variable_count);']);
        variable_count=variable_count+1;
    else
        eval([var_list{i} '=fixed_parameters(fixed_count);'])
        fixed_count=fixed_count+1;
    end
end

% Transforms
if is_variable(5)
    T=2*(exp(T)/(1+exp(T)))-1;
end
if is_variable(6)
    H(1,1)=exp(H(1,1));
end
if is_variable(7)
    H(2,2)=exp(H(2,2));
end
if is_variable(8)
    Q=exp(Q);
end
% mu
% Z
% T
% H
% Q
[a,v,LLF,l,P]=kf_bivariate_bansalyaron(y,mu,Z,T,R,H,Q,priorMean,priorCovariance);
l=-l;
LLF=-LLF;